with Ada.Strings.Unbounded;

with P42.Broadcast.Directory;
pragma Elaborate_All (P42.Broadcast.Directory);
with P42.Broadcast.Ressources;
pragma Elaborate_All (P42.Broadcast.Ressources);

with Common_Types;

package Broadcast_Stuff is

   type Process_Enum is (Passive,
                         Posix_Memory_Handler, Common_Transport_Layer,
                         Aws_Shell, Sub_System_Coordinator,
                         Narval_Actor, ENX_Server,
                         Launcher);
   type Process_Entry_Type (Kind : Process_Enum) is record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      case Kind is
         when Passive =>
            null;
         when Posix_Memory_Handler .. Launcher =>
            Host : Ada.Strings.Unbounded.Unbounded_String;
            Port : Positive;
      end case;
   end record;
   procedure Set_Name (Process_Entry : in out Process_Entry_Type;
                       Name : String);
   procedure Set_Port (Process_Entry : in out Process_Entry_Type;
                       Port : Positive)
     with Pre => Process_Entry.Kind /= Passive;
   procedure Set_Host (Process_Entry : in out Process_Entry_Type;
                       Host : String)
     with Pre => Process_Entry.Kind /= Passive;

   type Basic_Ressource_Type is tagged record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Host_Name : Ada.Strings.Unbounded.Unbounded_String;
      Process_Name : Ada.Strings.Unbounded.Unbounded_String;
      Error : Boolean;
   end record;
   function Get_Name (Item : Basic_Ressource_Type) return String is
      (Ada.Strings.Unbounded.To_String (Item.Name));

   type Buffer_Qualification_Enum is (All_Are_Ok);
   type Buffer_Ressource_Type (Kind : Buffer_Qualification_Enum) is new Basic_Ressource_Type with record
      Served : Boolean := False;
      Size : Common_Types.Buffer_Size_Type;
      Depth : Positive;
   end record;

   type Transport_Layer_Qualification_Enum is (Transport_Layer_TCP);
   type Transport_Layer_Ressource_Type (Kind : Transport_Layer_Qualification_Enum) is
     new  Basic_Ressource_Type with record
        Port : Positive;
        Qualified_Name : Ada.Strings.Unbounded.Unbounded_String;
   end record;

   Process_Group : constant String := "234.42.0.42";
   Process_Group_Port : constant := 4242;

   Buffer_Group : constant String := "234.0.42.42";
   Buffer_Group_Port : constant := 4243;

   TL_Group : constant String := "234.42.42.42";
   TL_Group_Port : constant := 4342;

   package Broadcast_Process is new P42.Broadcast (Process_Group,
                                                   Process_Group_Port);
   package Process_Directory is new Broadcast_Process.Directory (Process_Enum,
                                                                 Process_Entry_Type);

   package Broadcast_Buffer is new P42.Broadcast (Buffer_Group,
                                                  Buffer_Group_Port);
   package Buffer_Ressource is new Broadcast_Buffer.Ressources (Buffer_Qualification_Enum,
                                                                Buffer_Ressource_Type,
                                                                Get_Name);

   package Broadcast_TL is new P42.Broadcast (TL_Group,
                                              TL_Group_Port);
   package TL_Ressource is new Broadcast_TL.Ressources (Transport_Layer_Qualification_Enum,
                                                        Transport_Layer_Ressource_Type,
                                                        Get_Name);

   function Get_Process (Directory : in out Broadcast_Stuff.Process_Directory.Directory_Type;
                         Kind : Process_Enum;
                         Name : String;
                         Host_Name : String;
                         Watchdog : Natural := 5) return String;

   Process_Not_Found : exception;
   Ressource_Too_Long_To_Answer : exception;

end Broadcast_Stuff;
