with Ada.Containers.Indefinite_Vectors;

package Common_Types is

   pragma Preelaborate;
   pragma Remote_Types;

   --  buffer section
   type Buffer_Size_Type is mod 2 ** 32;
   type Buffer_Configuration_Type is record
      Buffer_Number : Positive;
      Size : Buffer_Size_Type;
   end record;
   type Policy_Type is (Multiple_Producers,
                        Consume_Once,
                        Consume_On_Fly);
   type Mode_Type is (On_Fly_Refresh, No_Transmit);

   --  CTL section
   type Data_Transport_Kind_Type is (Server, Client);
   type Data_Transport_Description_Type (Kind : Data_Transport_Kind_Type;
                                         Name_Length : Positive;
                                         Other_Length : Positive)
      is record
      Buffer_Name : String (1 .. Name_Length);
      case Kind is
         when Server =>
            Network_Interface : String (1 .. Other_Length);
            Port : Positive;
         when Client =>
            Host_Name : String (1 .. Other_Length);
      end case;
   end record;
   package DT_Descrition_Vectors is new Ada.Containers.Indefinite_Vectors
     (Positive,
      Data_Transport_Description_Type,
      "=");

   --  narval section
   type Acquisition_State_Type is (Initial, Configured, Loaded, Stopped,
                                   Running, Error);
   subtype Actor_State_Type is Acquisition_State_Type range Loaded .. Error;
   type Action is (Full_Reset, Partial_Reset, Actor_Reset,
                   Configure, Unconfigure,
                   Load, Unload,
                   Initialise, Reset_Com,
                   Start, Stop);
   type Action_Array is array (Positive range <>) of Action;

end Common_Types;
