with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;

package body Global_Options is

   use Ada.Strings.Unbounded;

   Name : Unbounded_String;
   Config_Name : Unbounded_String;
   Uuid : Unbounded_String;
   Port : Natural := 0;
   Automatic_Script_Loading : Boolean := False;

   function Get_Uuid return String is
   begin
      return To_String (Uuid);
   end Get_Uuid;

   function Get_Name return String is
   begin
      return To_String (Name);
   end Get_Name;

   function Get_Config_Name return String is
   begin
      return To_String (Config_Name);
   end Get_Config_Name;

   function Get_Port return Natural is
   begin
      return Port;
   end Get_Port;

   function Auto_Option return Boolean is
   begin
      return Automatic_Script_Loading;
   end Auto_Option;

   --  This procedure is called for each option of the command line.
   procedure Option_Callback (Switch : String;
                              Parameter : String;
                              Section : String);
   procedure Option_Callback (Switch : String;
                              Parameter : String;
                              Section : String) is
   begin
      Ada.Text_IO.Put_Line ("Switch : " & Switch);
      Ada.Text_IO.Put_Line ("Parameter : " & Parameter);
      Ada.Text_IO.Put_Line ("Section : " & Section);
      if Switch = "--name" or Switch = "-n" then
         Name := To_Unbounded_String (Parameter);
      elsif Switch = "--config_name" or Switch = "-c" then
         Config_Name := To_Unbounded_String (Parameter);
      elsif Switch = "--port" or Switch = "-p" or
        Switch = "--aws_server_port" or
        Switch = "-aws"
      then
         Port := Natural'Value (Parameter);
      elsif Switch = "--automatic-script-loading" or Switch = "-auto" then
         Automatic_Script_Loading := Boolean'Value (Parameter);
      elsif Switch = "--uuid" or Switch = "-u" then
         Uuid := To_Unbounded_String (Parameter);
      end if;
   end Option_Callback;

   Config : GNAT.Command_Line.Command_Line_Configuration;
   use GNAT.Command_Line;
begin
   Set_Usage (Config, "[switches]");
   Define_Switch (Config,
                  Switch => "-n=",
                  Long_Switch => "--name=",
                  Help => "DCOD Object name");
   Define_Switch (Config,
                  Switch => "-c=",
                  Long_Switch => "--config_name=",
                  Help => "DCOD config name");
   Define_Switch (Config,
                  Switch => "-p=",
                  Long_Switch => "--port=",
                  Help => "DCOD HTTP port");
   Define_Switch (Config,
                  Switch => "-aws=",
                  Long_Switch => "--aws_server_port=",
                  Help => "Aws Server Port, here for compatibility reasons");
   Define_Switch (Config,
                  Switch => "-hook=",
                  Long_Switch => "--hook_end_point=",
                  Help => "DCOD Hook end point");
   Define_Switch (Config,
                  Switch => "-auto=",
                  Long_Switch => "--automatic-script-loading=",
                  Help => "automatic script loading");
   Define_Switch (Config,
                  Switch => "-u=",
                  Long_Switch => "--uuid=",
                  Help => "uuid associated to the process");
   Getopt (Config, Option_Callback'Access);
   Free (Config);
end Global_Options;

