with Ada.Exceptions;
with Ada.Streams;
with Ada.Text_IO;
with Interfaces;

with Low_Level_Network.Utils;
with Network_Utils;
with Data_Transport.Socket_Utilities;

package body Data_Transport.Socket_Server is

   -------------------------
   -- Socket_Server_Task --
   -------------------------

   procedure Send_Data (Socket : GNAT.Sockets.Socket_Type;
                        Buffer_Set : Buffers.Buffer_Consume_Access);

   task body Socket_Server_Task is
      use type Buffers.Buffer_Consume_Access;
      Server : GNAT.Sockets.Socket_Type;
      Socket : GNAT.Sockets.Socket_Type;
      Address : GNAT.Sockets.Sock_Addr_Type;
      use Low_Level_Network.Utils;
   begin
      --  Ada.Text_IO.Put_Line ("socket emitter step 1");
      select
         accept Initialise
           (Network_Interface : String;
            Port : in GNAT.Sockets.Port_Type) do
            Address.Addr := GNAT.Sockets.Inet_Addr
              (Get_Address (Network_Interface));
            declare
               use GNAT.Sockets;
            begin
               Address.Port := Port;
               Create_Socket (Server);
               --  Set_Socket_Option (Server, Option => (Name => Send_Timeout,
               --                                        Timeout => 0.02));
               Set_Socket_Option (Server, Option => (Name => Reuse_Address,
                                                     Enabled => True));
               Bind_Socket (Server, Address);
               Listen_Socket (Server);
            end;
         end Initialise;
      or
         terminate;
      end select;
      --  Ada.Text_IO.Put_Line ("socket emitter step 2");
      loop
         select
            accept Connect;
            --  Ada.Text_IO.Put_Line ("before connect");
            GNAT.Sockets.Accept_Socket (Server, Socket, Address);
            --  Ada.Text_IO.Put_Line ("after connect");
            exit;
         or
            terminate;
         end select;
      end loop;
      --  Ada.Text_IO.Put_Line ("socket emitter step 3");
      loop
         select
            accept Disconnect;
            --  Ada.Text_IO.Put_Line ("just accepted disconnect");
            declare
               Data : constant Ada.Streams.Stream_Element_Array (1 .. 4) :=
                 (others => 0);
            begin
               --  Ada.Text_IO.Put_Line
               --    ("DT socket produce send 0 for disconnection");
               Socket_Utilities.Send (Socket, Data);
            end;
            GNAT.Sockets.Shutdown_Socket (Socket);
            GNAT.Sockets.Shutdown_Socket (Server);
            exit;
         else
            select
               Buffer_Set.Block_Full;
               --  Ada.Text_IO.Put_Line ("block full seen");
               Send_Data (Socket, Buffer_Set);
               --  Ada.Text_IO.Put_Line ("block full sent");
            or
               delay 0.1;
            end select;
         end select;
      end loop;
      --  Ada.Text_IO.Put_Line ("socket emitter step 4");
   exception
      when E : others =>
         Ada.Text_IO.Put_Line ("task socket emitter unattended exception : " &
                               Ada.Exceptions.Exception_Name (E) & "," &
                               Ada.Exceptions.Exception_Message (E));
   end Socket_Server_Task;

   procedure Send_Data (Socket : GNAT.Sockets.Socket_Type;
                        Buffer_Set : Buffers.Buffer_Consume_Access) is
      use type Buffers.Buffer_Size_Type;
      use Network_Utils;
      Buffer_Size : constant Interfaces.Unsigned_32 :=
        To_Network (Interfaces.Unsigned_32 (Buffer_Set.Full_Size));
   begin
      --  Ada.Text_IO.Put_Line ("send data" & Buffer_Set.Full_Size'Img);
      declare
         Data : Ada.Streams.Stream_Element_Array (1 .. 4);
         for Data'Address use Buffer_Size'Address;
      begin
         Socket_Utilities.Send (Socket, Data);
      exception
         when E : others =>
            Ada.Text_IO.Put_Line
              ("size message transmit unattended exception : " &
                 Ada.Exceptions.Exception_Name (E) & "," &
                 Ada.Exceptions.Exception_Message (E));
            raise;
      end;
      declare
         Buffer_Handle : Buffers.Buffer_Handle_Type;
      begin
         Buffer_Set.Get_Full_Buffer (Buffer_Handle);
         declare
            Data_Size : constant Ada.Streams.Stream_Element_Offset :=
              Ada.Streams.Stream_Element_Offset
              (Buffers.Get_Used_Bytes (Buffer_Handle));
            Data : Ada.Streams.Stream_Element_Array (1 .. Data_Size);
            for Data'Address use Buffers.Get_Address (Buffer_Handle);
         begin
            Socket_Utilities.Send (Socket, Data);
         exception
            when E : others =>
               Ada.Text_IO.Put_Line
                 ("data transmit unattended exception : " &
                    Ada.Exceptions.Exception_Name (E) & "," &
                    Ada.Exceptions.Exception_Message (E));
               raise;
         end;
         Buffer_Set.Release_Full_Buffer (Buffer_Handle);
      end;
   end Send_Data;

end Data_Transport.Socket_Server;
