with Ada.Text_IO;

package body Bufferized_Data_Transport is

   use Ada.Strings.Unbounded;

   protected body Transport_Layer_Shell is
      procedure Request_Server_First_Stage (Name : String;
                                            Buffer_Name : String) is
      begin
         Transport_Layers.Append ((Kind => Server,
                                   Name => To_Unbounded_String (Name),
                                   Buffer_Name => To_Unbounded_String (Buffer_Name),
                                   TL_Units => <>));
      end Request_Server_First_Stage;

      procedure Request_Server_Second_Stage (Name : String;
                                             Qualified_Name : String;
                                             Consumer : Buffers.Shared.Consume.Consume_Couple_Access;
                                             Server_Socket : Data_Transport.Socket_Server.Socket_Server_Access) is
      begin
         for TL_Container of Transport_Layers loop
            if TL_Container.Name = Name and TL_Container.Kind = Server then
               TL_Container.TL_Units.Append ((Consumer => Consumer,
                                              Server_Socket => Server_Socket,
                                              Qualified_Name => To_Unbounded_String (Qualified_Name)));
            end if;
         end loop;
      end Request_Server_Second_Stage;

      procedure Request_Client (Name : String;
                               Buffer_Name : String;
                               Server_Name : String;
                               Host_Name : Ada.Strings.Unbounded.Unbounded_String;
                               Qualified_Name : Ada.Strings.Unbounded.Unbounded_String;
                               Producer : Buffers.Shared.Produce.Produce_Couple_Access;
                               Client_Socket : Data_Transport.Socket_Client.Socket_Client_Access) is
      begin
         Transport_Layers.Append ((Kind => Client,
                                   Name => To_Unbounded_String (Name),
                                   Buffer_Name => To_Unbounded_String (Buffer_Name),
                                   Server_Name => To_Unbounded_String (Server_Name),
                                   Host_Name => Host_Name,
                                   Qualified_Name => Qualified_Name,
                                   Producer => Producer,
                                   Client_Socket => Client_Socket));
      end Request_Client;

      procedure Release_Server (Server_Name : String;
                               Consumer : out Buffers.Shared.Consume.Consume_Couple_Access;
                               Server_Socket : out Data_Transport.Socket_Server.Socket_Server_Access) is
         Cursor : Data_Transport_Container_Vectors.Cursor;
         use type Data_Transport_Container_Vectors.Cursor;
      begin
         Ada.Text_IO.Put_Line ("BDT delete server from output : enter " & Server_Name);
         Consumer := null;
         Server_Socket := null;
         Cursor := Transport_Layers.First;
         Main_Loop :
         loop
            exit Main_Loop when Cursor = Data_Transport_Container_Vectors.No_Element;
            Ada.Text_IO.Put_Line ("delete TL step 1");
            declare
               TL_Container : Data_Transport_Container_Type :=
                 Data_Transport_Container_Vectors.Element (Cursor);
            begin
               if TL_Container.Name = Server_Name and TL_Container.Kind = Server then
                  for TL_Unit of TL_Container.TL_Units loop
                     Consumer := TL_Unit.Consumer;
                     Server_Socket := TL_Unit.Server_Socket;
                  end loop;
                  TL_Container.TL_Units.Clear;
                  Ada.Text_IO.Put_Line ("delete TL step 2 ");
                  Transport_Layers.Delete (Cursor);
                  Ada.Text_IO.Put_Line ("delete TL step 3");
                  exit Main_Loop;
               else
                  Cursor := Data_Transport_Container_Vectors.Next (Cursor);
               end if;
            end;
         end loop Main_Loop;
         Ada.Text_IO.Put_Line ("BDT delete server from output : exit");
      end Release_Server;

      procedure Release_Client (Name : String;
                              Host_Name : out Ada.Strings.Unbounded.Unbounded_String;
                              Qualified_Name : out Ada.Strings.Unbounded.Unbounded_String;
                              Producer : out Buffers.Shared.Produce.Produce_Couple_Access;
                              Client_Socket : out Data_Transport.Socket_Client.Socket_Client_Access) is
         Cursor : Data_Transport_Container_Vectors.Cursor;
         use type Data_Transport_Container_Vectors.Cursor;
      begin
         Ada.Text_IO.Put_Line ("BDT delete input : enter");
         Producer := null;
         Client_Socket := null;
         Cursor := Transport_Layers.First;
         loop
            exit when Cursor = Data_Transport_Container_Vectors.No_Element;
            Ada.Text_IO.Put_Line ("delete TL step 1");
            declare
               TL_Container : constant Data_Transport_Container_Type :=
                 Data_Transport_Container_Vectors.Element (Cursor);
            begin
               if TL_Container.Name = Name and TL_Container.Kind = Client then
                  Ada.Text_IO.Put_Line ("delete TL step 2 ");
                  Producer := TL_Container.Producer;
                  Client_Socket := TL_Container.Client_Socket;
                  Host_Name := TL_Container.Host_Name;
                  Qualified_Name := TL_Container.Qualified_Name;
                  Transport_Layers.Delete (Cursor);
                  Ada.Text_IO.Put_Line ("delete TL step 3");
                  exit;
               else
                  Cursor := Data_Transport_Container_Vectors.Next (Cursor);
               end if;
            end;
         end loop;
         Ada.Text_IO.Put_Line ("BDT delete input : exit");
      end Release_Client;

      function Server_Exist (Name : String) return Boolean is
      begin
         for Transport_Layer of Transport_Layers loop
            if Transport_Layer.Name = Name and Transport_Layer.Kind = Server then
               return True;
            end if;
         end loop;
         return False;
      end Server_Exist;

      function Client_Exist (Name : String) return Boolean is
      begin
         for Transport_Layer of Transport_Layers loop
            if Transport_Layer.Name = Name and Transport_Layer.Kind = Client then
               return True;
            end if;
         end loop;
         return False;
      end Client_Exist;

   end Transport_Layer_Shell;

end Bufferized_Data_Transport;
