with Ada.Containers.Indefinite_Vectors;
with Ada.Containers.Vectors;
with Ada.Strings.Unbounded;

with Buffers.Shared.Consume;
with Buffers.Shared.Produce;

with Data_Transport.Socket_Client;
with Data_Transport.Socket_Server;

package Bufferized_Data_Transport is

   type Kind_Enum is (Client, Server);

   type TL_Unit_Type is record
      Consumer : Buffers.Shared.Consume.Consume_Couple_Access;
      Server_Socket : Data_Transport.Socket_Server.Socket_Server_Access;
      Qualified_Name : Ada.Strings.Unbounded.Unbounded_String;
   end record;
   package TL_Unit_Vectors is new Ada.Containers.Vectors (Positive,
                                                          TL_Unit_Type,
                                                          "=");

   type Data_Transport_Container_Type (Kind : Kind_Enum) is record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Buffer_Name : Ada.Strings.Unbounded.Unbounded_String;
      case Kind is
         when Client =>
            Server_Name : Ada.Strings.Unbounded.Unbounded_String;
            Host_Name : Ada.Strings.Unbounded.Unbounded_String;
            Qualified_Name : Ada.Strings.Unbounded.Unbounded_String;
            Producer : Buffers.Shared.Produce.Produce_Couple_Access;
            Client_Socket : Data_Transport.Socket_Client.Socket_Client_Access;
         when Server =>
            TL_Units : TL_Unit_Vectors.Vector;
      end case;
   end record;
   package Data_Transport_Container_Vectors is new Ada.Containers.Indefinite_Vectors
     (Positive, Data_Transport_Container_Type, "=");

   protected Transport_Layer_Shell is
      procedure Request_Server_First_Stage (Name : String;
                                            Buffer_Name : String);
      procedure Request_Server_Second_Stage (Name : String;
                                             Qualified_Name : String;
                                             Consumer : Buffers.Shared.Consume.Consume_Couple_Access;
                                             Server_Socket : Data_Transport.Socket_Server.Socket_Server_Access);

      procedure Request_Client (Name : String;
                               Buffer_Name : String;
                               Server_Name : String;
                               Host_Name : Ada.Strings.Unbounded.Unbounded_String;
                               Qualified_Name : Ada.Strings.Unbounded.Unbounded_String;
                               Producer : Buffers.Shared.Produce.Produce_Couple_Access;
                               Client_Socket : Data_Transport.Socket_Client.Socket_Client_Access);

      procedure Release_Server (Server_Name : String;
                               Consumer : out Buffers.Shared.Consume.Consume_Couple_Access;
                               Server_Socket : out Data_Transport.Socket_Server.Socket_Server_Access);

      procedure Release_Client (Name : String;
                              Host_Name : out Ada.Strings.Unbounded.Unbounded_String;
                              Qualified_Name : out Ada.Strings.Unbounded.Unbounded_String;
                              Producer : out Buffers.Shared.Produce.Produce_Couple_Access;
                              Client_Socket : out Data_Transport.Socket_Client.Socket_Client_Access);

      function Server_Exist (Name : String) return Boolean;

      function Client_Exist (Name : String) return Boolean;
   private
      Transport_Layers : Data_Transport_Container_Vectors.Vector;
   end Transport_Layer_Shell;

end Bufferized_Data_Transport;
