with Ada.Exceptions;
with Ada.Text_IO;

with GNAT.Traceback.Symbolic;

with AWS.Server;
with AWS.Config.Set;

with SOAP.Dispatchers.Callback;

with Global_Options;
with DCOD.Ctl.Http;
with DCOD.Ctl.Soap_Cb;

with Broadcast_Stuff;
with Broadcast_TLs;
with Ctl_Directory;

procedure Ctl_Stand_Alone is
   AWS_Server : AWS.Server.HTTP;
   Config : AWS.Config.Object;
begin
   AWS.Config.Set.Server_Name (Config, "Common Transport Layer");
   AWS.Config.Set.Reuse_Address (Config, True);
   Broadcast_Stuff.Set_Name (Ctl_Directory.An_Entry, Global_Options.Get_Name);
   if Global_Options.Get_Port = 0 then
      --  Default AWS port is 5678
      AWS.Config.Set.Server_Port (Config, 8765);
      Broadcast_Stuff.Set_Port (Ctl_Directory.An_Entry, 8765);
   else
      AWS.Config.Set.Server_Port (Config, Global_Options.Get_Port);
      Broadcast_Stuff.Set_Port (Ctl_Directory.An_Entry, Global_Options.Get_Port);
   end if;
   Ctl_Directory.Start;
   AWS.Server.Start (AWS_Server,
                     Dispatcher => SOAP.Dispatchers.Callback.Create
                       (DCOD.Ctl.Http.Callback'Access,
                        DCOD.Ctl.Soap_Cb.Callback'Access),
                     Config => Config);
   AWS.Server.Wait (AWS.Server.Forever);
   Ctl_Directory.Stop;
   Broadcast_TLs.Close;
   Ada.Text_IO.Put_Line ("out of CTL " & Global_Options.Get_Name);
exception
   when E : others =>
      Ctl_Directory.Stop;
      --  Broadcast_Buffers.Close;
      Broadcast_TLs.Close;
      Ada.Text_IO.Put_Line ("   exception name : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("exception message : " & Ada.Exceptions.Exception_Message (E));
      Ada.Text_IO.Put_Line (GNAT.Traceback.Symbolic.Symbolic_Traceback (E));
end Ctl_Stand_Alone;
