with Data_Transport.Socket_Client;
with Buffers.Shared.Produce;
with GNAT.Sockets;
with GNAT.Command_Line;
with Ada.Text_IO;
with Ada.Strings.Unbounded;

with Utiles_Task;

procedure Test_Transport_Shared_Consume is
   Production : Buffers.Shared.Produce.Produce_Couple_Type;
   Buffer : Buffers.Shared.Produce.Produce_Type renames Production.Producer;
   Client : Data_Transport.Socket_Client.Socket_Client_Task
     (Buffer'Unchecked_Access);
   Options : constant String := "buffer_name= host_name= port= verbose quiet";
   use Ada.Strings.Unbounded;
   Host_Name : Unbounded_String := To_Unbounded_String ("localhost");
   Buffer_Name : Unbounded_String := To_Unbounded_String ("green_buffer");
   Port : GNAT.Sockets.Port_Type := 8042;
   Verbose : Boolean := True;
   A_Task_Communication : aliased Utiles_Task.Task_Communication;
   A_Terminate_Task : Utiles_Task.Terminate_Task (A_Task_Communication'Access);
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'b' =>
            Buffer_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'h' =>
            Host_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Port := GNAT.Sockets.Port_Type'Value (GNAT.Command_Line.Parameter);
         when 'q' =>
            Verbose := False;
         when 'v' =>
            Verbose := True;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   Buffer.Set_Name (To_String (Buffer_Name));
   Production.Message_Handling.Start (1.0);
   select
      Buffer.Is_Initialised;
   or
      delay 3.0;
      Ada.Text_IO.Put_Line ("too long to initialise");
      Production.Message_Handling.Stop;
      Buffer.Delete;
      return;
   end select;
   Client.Initialise (To_String (Host_Name), Port);
   Client.Connect;
   loop
      select
         A_Task_Communication.Can_Stop;
         exit;
      or
         delay 1.0;
         if Verbose then
            Ada.Text_IO.Put_Line ("waiting");
            Ada.Text_IO.Put_Line ("status " & Buffer.Get_State);
         end if;
      end select;
   end loop;
   if Verbose then
      Ada.Text_IO.Put_Line ("before disconnect");
   end if;
   Client.Disconnect;
   if Verbose then
      Ada.Text_IO.Put_Line ("after disconnect");
   end if;
   Production.Message_Handling.Stop;
   if Verbose then
      Ada.Text_IO.Put_Line ("after stop task");
   end if;
   Buffer.Delete;
   if Verbose then
      Ada.Text_IO.Put_Line ("producer end");
   end if;
end Test_Transport_Shared_Consume;
