with Ada.Exceptions;
with Ada.Strings.Unbounded;
with Data_Transport.Socket_Server;
with Buffers.Shared.Consume;
with GNAT.Sockets;
with GNAT.Command_Line;
with Ada.Text_IO;

with Utiles_Task;

procedure Test_Transport_Shared_Produce is
   Consumption : Buffers.Shared.Consume.Consume_Couple_Type;
   Buffer : Buffers.Shared.Consume.Consume_Type renames Consumption.Consumer;
   Server : Data_Transport.Socket_Server.Socket_Server_Task
     (Buffer'Unchecked_Access);
   Port : GNAT.Sockets.Port_Type := 8042;
   use Ada.Strings.Unbounded;
   Buffer_Name : Unbounded_String := To_Unbounded_String ("green_buffer");
   Options : constant String := "buffer_name= port= interface= verbose quiet";
   Interface_Host : Unbounded_String := To_Unbounded_String ("lo");
   Verbose : Boolean := True;
   A_Task_Communication : aliased Utiles_Task.Task_Communication;
   A_Terminate_Task : Utiles_Task.Terminate_Task (A_Task_Communication'Access);
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'b' =>
            Buffer_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'i' =>
            Interface_Host := To_Unbounded_String
              (GNAT.Command_Line.Parameter);
         when 'p' =>
            Port := GNAT.Sockets.Port_Type'Value (GNAT.Command_Line.Parameter);
         when 'q' =>
            Verbose := False;
         when 'v' =>
            Verbose := True;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   Buffer.Set_Name (To_String (Buffer_Name));
   Consumption.Message_Handling.Start (1.0);
   Server.Initialise (To_String (Interface_Host), Port);
   if Verbose then
      Ada.Text_IO.Put_Line ("Get port :" & Port'Img);
   end if;
   Server.Connect;
   if Verbose then
      Ada.Text_IO.Put_Line ("Connect done");
   end if;
   loop
      select
         A_Task_Communication.Can_Stop;
         exit;
      or
         delay 1.0;
         if Verbose then
            Ada.Text_IO.Put_Line ("waiting");
            Ada.Text_IO.Put_Line ("status " & Buffer.Get_State);
         end if;
      end select;
   end loop;
   if Verbose then
      Ada.Text_IO.Put_Line ("out of waiting loop");
      Ada.Text_IO.Put_Line ("Disconnect");
   end if;
   Server.Disconnect;
   Consumption.Message_Handling.Stop;
   if Verbose then
      Ada.Text_IO.Put_Line ("exit : test");
   end if;
   Buffer.Delete;
exception
   when E : others =>
      Ada.Text_IO.Put_Line ("Exception Received : " &
                              Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("Exception message : " &
                              Ada.Exceptions.Exception_Message (E));
      Buffer.Delete;
      Server.Disconnect;
end Test_Transport_Shared_Produce;
