with Ada.Command_Line;
with Ada.Exceptions;
with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;
with GNAT.Sockets;

with Broadcast_Stuff;
with Passive_Directory;

with Dcod_Pmh_Service.Client;

procedure Ddp_Instanciate is
   Options : constant String := "buffer= help";
   Buffer_Name : Ada.Strings.Unbounded.Unbounded_String;
   use Ada.Strings.Unbounded;
   Non_Conformant_Call : exception;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'b' =>
            Buffer_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'h' =>
            Ada.Text_IO.Put_Line ("usage :");
            Ada.Text_IO.Put_Line (Ada.Command_Line.Command_Name & " {-buffer=buffer_name|-help}");
            return;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if Buffer_Name = Null_Unbounded_String then
      raise Non_Conformant_Call with "one has to set -buffer option in order call this test code";
   end if;
   Ada.Text_IO.Put_Line ("step 1");
   Passive_Directory.Start;
   declare
      Pmh_Endpoing : constant String := Broadcast_Stuff.Get_Process (Passive_Directory.Directory,
                                                                     Broadcast_Stuff.Posix_Memory_Handler,
                                                                     "stand_alone",
                                                                     GNAT.Sockets.Host_Name);
   begin
      Ada.Text_IO.Put_Line ("pmh end point : " & Pmh_Endpoing);
      Dcod_Pmh_Service.Client.Request_Buffer (To_String (Buffer_Name), Endpoint => Pmh_Endpoing);
   end;
   Ada.Text_IO.Put_Line ("step 2");
   Passive_Directory.Stop;
   Ada.Text_IO.Put_Line ("step 3");
exception
   when Broadcast_Stuff.Process_Not_Found =>
      Ada.Text_IO.Put_Line ("no pmh found with name stand_alone");
      Passive_Directory.Stop;
   when Non_Conformant_Call =>
      Passive_Directory.Stop;
      raise;
   when E : others =>
      Ada.Text_IO.Put_Line ("unexpected exception : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("                       " & Ada.Exceptions.Exception_Message  (E));
      Passive_Directory.Stop;
end Ddp_Instanciate;
