with Ada.Strings.Unbounded;

package Buffers.Local is

   protected type Local_Buffer_Type is new Buffer_Host_Interface and
     Buffer_Consume_Interface and
     Buffer_Produce_Interface with
      procedure Set_Name (Name : String);
      function Get_Name return String;
      function Get_State return String;
      function No_Data return Boolean;
      function Compare (Name : String) return Boolean;
      procedure Reset;
      procedure Delete;
      function Safe_Delete return Boolean;
      function Get_Buffers_Size return Buffer_Size_Type;
      procedure Initialise (Buffer_Number : Positive;
                            Size : Buffer_Size_Type);
      entry Get_Free_Buffer (Buffer_Handle : in out Buffer_Handle_Type'Class);
      entry Block_Free;
      procedure Release_Free_Buffer
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      procedure Release_Free_Buffer_Not_Used
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      function Free_Is_Empty return Boolean;
      function Free_Occupancy_Rate return Natural;
      function Free_Depth return Positive;
      entry Get_Full_Buffer (Buffer_Handle : in out Buffer_Handle_Type'Class);
      entry Block_Full;
      procedure Release_Full_Buffer
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      procedure Release_Full_Buffer_Not_Used
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      function Full_Is_Empty return Boolean;
      function Full_Size return Buffer_Size_Type;
      overriding
      procedure Set_Policy (Policy : Common_Types.Policy_Type;
                            Value : Boolean);
      overriding
      procedure Set_Mode (Mode : Common_Types.Mode_Type;
                          Value : Boolean);
      --  This function gives back the policies of the buffer.
      overriding
      function Get_Policies return Policy_Array;
      --  This function gives the policies supported by the buffer.
      overriding
      function Get_Available_Policies return Policy_Array;
      overriding
      function Is_In_Error_State return Boolean;
      overriding
      entry Is_Initialised;
   private
      Modes : Mode_Array := (others => False);
      Policies : Policy_Array := (others => False);
      Name : Ada.Strings.Unbounded.Unbounded_String :=
        Ada.Strings.Unbounded.Null_Unbounded_String;
      Free_Buffers : Vector;
      Full_Buffers : Vector;
      Buffer_Size : Buffer_Size_Type;
      Initialised : Boolean := False;
      Initial_Depth : Positive := 1;
      Error_State : Boolean := False;
   end Local_Buffer_Type;

   type Local_Buffer_Access is access all Local_Buffer_Type;
   procedure Free is new Ada.Unchecked_Deallocation (Local_Buffer_Type,
                                                     Local_Buffer_Access);

private

   Available_Policies : constant Policy_Array :=
     (Common_Types.Multiple_Producers => True,
      Common_Types.Consume_Once => False,
      Common_Types.Consume_On_Fly => False);

end Buffers.Local;
