with Ada.Strings.Unbounded;

with Buffers.Shared.Produce;
with Buffers.Shared.Consume;

package Buffers.Mailboxes is

   --  ATTENTION mailbox size first version is 4096 - depth 16

   MAILBOX_SIZE : constant := 4096;
   MAILBOX_DEPTH : constant := 16;

   type Mailbox_Send_Type is limited record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Associated_Pmh : Ada.Strings.Unbounded.Unbounded_String;
      Production : Shared.Produce.Produce_Couple_Type;
   end record;

   type Mailbox_Receive_Type is limited record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Associated_Pmh : Ada.Strings.Unbounded.Unbounded_String;
      Consumption : Shared.Consume.Consume_Couple_Type;
   end record;

   procedure Open_Send (Mailbox : out Mailbox_Send_Type;
                        Name : String;
                        Pmh : String);
   procedure Open_Receive (Mailbox : out Mailbox_Receive_Type;
                           Name : String;
                           Pmh : String);

   Initialised_Too_Long : exception;

   procedure Close (Mailbox : in out Mailbox_Send_Type);
   procedure Close (Mailbox : in out Mailbox_Receive_Type);

   Get_Buffer_Too_Long : exception;

   generic
      type Message_Type (<>) is private;
   procedure Send_Generic (Mailbox : in out Mailbox_Send_Type;
                           Message : Message_Type);

   generic
      type Message_Type (<>) is private;
   function Receive_Generic (Mailbox : in out Mailbox_Receive_Type) return Message_Type;

end Buffers.Mailboxes;
