with Ada.Containers.Vectors;
with Ada.Strings.Unbounded;
with Ada.Unchecked_Deallocation;

with POSIX.IO;

package Buffers.Shared.Consume is

   protected type Consume_Type is new Buffer_Consume_Interface with
      entry Get_Full_Buffer (Buffer_Handle : in out Buffer_Handle_Type'Class);
      entry Block_Full;
      procedure Release_Full_Buffer
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      procedure Release_Full_Buffer_Not_Used
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      function Full_Is_Empty return Boolean;
      function Full_Size return Buffer_Size_Type;
      procedure Set_Name (Name : String);
      function Get_Name return String;
      function Get_State return String;
      function No_Data return Boolean;
      function Compare (Name : String) return Boolean;
      procedure Reset;
      procedure Delete;
      function Safe_Delete return Boolean;
      function Get_Buffers_Size return Buffer_Size_Type;
      procedure Append_Buffer (Buffer : Buffer_Description_Type);
      function Get_Uuid return Uuid_Image_Type;
      procedure Set_Size (Size : Buffer_Size_Type);
      procedure Full_Available (Message : Message_Type);
      function Get_Memory_Descriptor return POSIX.IO.File_Descriptor;
      overriding
      function Is_In_Error_State return Boolean;
      overriding
      entry Is_Initialised;
      procedure Error_Message;
      procedure Set_On_Fly_Flag;
      function On_Fly_Set return Boolean;
      procedure No_More_Buffers_Message;
      procedure Set_Host_Available (New_Value : Boolean);
   private
      Host_Available : Boolean := False;
      Uuid_Image : Uuid_Image_Type := Null_Uuid;
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Server_Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Memory_Descriptor : POSIX.IO.File_Descriptor;
      Buffers : Vector;
      Full_Buffers : Vector;
      Buffer_Size : Buffer_Size_Type := 0;
      Initial_Depth : Natural := 0;
      No_Buffer_In_Use : Boolean := True;
      Ask_Buffer_Done : Boolean := False;
      Error_State : Boolean := False;
      Initialised : Boolean := False;
      On_Fly : Boolean := False;
   end Consume_Type;
   type Consume_Access is access all Consume_Type;

   type Consume_Couple_Type;

   task type Message_Handling_Task (Wrapper : access Consume_Couple_Type) is
      entry Start (Timeout : Duration);
      entry Stop;
   end Message_Handling_Task;

   type Consume_Couple_Type is limited record
      Consumer : aliased Consume_Type;
      Message_Handling : Message_Handling_Task (Consume_Couple_Type'Access);
   end record;
   type Consume_Couple_Access is access all Consume_Couple_Type;
   procedure Free is new Ada.Unchecked_Deallocation (Consume_Couple_Type,
                                                     Consume_Couple_Access);
   package Consumer_Vectors is new Ada.Containers.Vectors
     (Positive, Consume_Couple_Access, "=");
   procedure Clean (Vector : in out Consumer_Vectors.Vector);

end Buffers.Shared.Consume;
