with Ada.Strings.Unbounded;

with POSIX.IO;

package Buffers.Shared.Produce is

   protected type Produce_Type is new Buffer_Produce_Interface with
      entry Get_Free_Buffer (Buffer_Handle : in out Buffer_Handle_Type'Class);
      entry Block_Free;
      procedure Release_Free_Buffer
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      procedure Release_Free_Buffer_Not_Used
        (Buffer_Handle : in out Buffer_Handle_Type'Class);
      function Free_Is_Empty return Boolean;
      procedure Set_Name (Name : String);
      function Get_Name return String;
      function Get_State return String;
      function No_Data return Boolean;
      function Compare (Name : String) return Boolean;
      procedure Reset;
      procedure Delete;
      function Safe_Delete return Boolean;
      function Get_Buffers_Size return Buffer_Size_Type;
      function Free_Occupancy_Rate return Natural;
      overriding
      function Free_Depth return Positive;
      procedure Free_Available (Message : Message_Type);
      overriding
      function Is_In_Error_State return Boolean;
      overriding
      entry Is_Initialised;
      procedure No_More_Buffers_Message;
      procedure Append_Buffer (Buffer : Buffer_Description_Type);
      function Get_Uuid_Image return String;
      procedure Set_Size (Size : Buffer_Size_Type);
      function Get_Memory_Descriptor return POSIX.IO.File_Descriptor;
      procedure Error_Message;
      procedure Set_Host_Available (New_Value : Boolean);
   private
      Host_Available : Boolean := False;
      Uuid_Image : Uuid_Image_Type := Null_Uuid;
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Server_Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Memory_Descriptor : POSIX.IO.File_Descriptor;
      Buffers : Vector;
      Free_Buffers : Vector;
      Buffer_Size : Buffer_Size_Type := 0;
      Initial_Depth : Natural := 0;
      Initialised : Boolean := False;
      Error_State : Boolean := False;
      Ask_Buffer_Done : Boolean := False;
      Buffer_In_Use : Boolean := False;
   end Produce_Type;
   type Produce_Access is access all Produce_Type;

   type Produce_Couple_Type;

   task type Message_Handling_Task (Wrapper : access Produce_Couple_Type) is
      entry Start (Timeout : Duration);
      entry Stop;
   end Message_Handling_Task;

   type Produce_Couple_Type is limited record
      Producer : aliased Produce_Type;
      Message_Handling : Message_Handling_Task (Produce_Couple_Type'Access);
   end record;
   type Produce_Couple_Access is access all Produce_Couple_Type;
   procedure Free is new Ada.Unchecked_Deallocation (Produce_Couple_Type,
                                                     Produce_Couple_Access);
   package Producer_Vectors is new Ada.Containers.Vectors
     (Positive, Produce_Couple_Access, "=");
   procedure Clean (Vector : in out Producer_Vectors.Vector);

end Buffers.Shared.Produce;
