with Ada.Containers.Vectors;

with POSIX.Message_Queues;
with POSIX.C;

with P42.Messages_Queues;

package Buffers.Shared is

   Already_Initialised : exception;

   type Message_Kind_Type is (Subscribe_Consume,
                              Subscribe_Produce,
                              Subscribe_Consume_On_Fly,
                              Get_Free,
                              Get_Full,
                              Get_Spy,
                              Release_Free,
                              Release_Full,
                              Release_Spy,
                              New_Buffer_Available,
                              No_More_Buffers,
                              Free_Available,
                              Full_Available,
                              Spyed_Available,
                              Unsubscribe_Consume,
                              Unsubscribe_Produce,
                              Unsubscribe_Consume_On_Fly,
                              Propagate_Delete,
                              Error);
   for Message_Kind_Type'Size use 8;

   subtype Uuid_Image_Type is String (1 .. 36);
   Null_Uuid : constant Uuid_Image_Type := (others => ' ');

   Message_Size : constant := 40;
   type Message_Type (Kind : Message_Kind_Type) is record
      case Kind is
         when Propagate_Delete | Error | No_More_Buffers =>
            null;
         when Subscribe_Consume .. Get_Spy |
           Unsubscribe_Consume .. Unsubscribe_Consume_On_Fly =>
            Sender : Uuid_Image_Type := Null_Uuid;
         when others =>
            ID : Buffer_ID_Type;
            Bytes : Buffer_Size_Type;
      end case;
   end record;
   for Message_Type'Size use Message_Size * 8;

   package Communication_Queue_Pkg is new P42.Messages_Queues (Message_Type);

   procedure Send (Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
                   Message : Message_Type) renames Communication_Queue_Pkg.Send;
   function Receive (Queue : POSIX.Message_Queues.Message_Queue_Descriptor)
     return Message_Type renames Communication_Queue_Pkg.Receive;
   function Timed_Receive
     (Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Abs_Timeout : POSIX.C.struct_timespec)
     return Message_Type renames Communication_Queue_Pkg.Timed_Receive;
   function Relative_Timed_Receive
     (Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Relative_Timeout : Natural)
     return Message_Type renames Communication_Queue_Pkg.Relative_Timed_Receive;
   function Relative_Timed_Receive
     (Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Relative_Timeout : Duration)
     return Message_Type renames Communication_Queue_Pkg.Relative_Timed_Receive;

   type Unique_Queue_Type is record
      Queue : POSIX.Message_Queues.Message_Queue_Descriptor;
      Uuid : Uuid_Image_Type;
   end record;

   package Queue_Vectors is new Ada.Containers.Vectors
     (Positive, Unique_Queue_Type);
   function Find (Queue_Vector : Queue_Vectors.Vector;
                  Uuid : Uuid_Image_Type) return Unique_Queue_Type;
   procedure Find_And_Remove (Queue_Vector : in out Queue_Vectors.Vector;
                              Uuid : Uuid_Image_Type;
                              Queue : out Unique_Queue_Type);
   Uuid_Not_Found : exception;

   Incoherent_Buffers : exception;
   Not_Page_Aligned_Size : exception;
   Host_Not_Available : exception;

end Buffers.Shared;
