with Ada.Containers.Vectors;
with Ada.Strings.Unbounded;
with Ada.Unchecked_Deallocation;

with Common_Types;
with Common_Types.String_Vectors;
with Buffers.Shared.Host;

package Buffers.Shell is

   type Buffer_Container_Type
     (Message_Handling : Buffers.Shared.Host.Message_Handling_Access) is record
      Name : Ada.Strings.Unbounded.Unbounded_String;
      Buffer : aliased Buffers.Shared.Host.Host_Type (Message_Handling);
   end record;
   type Buffer_Container_Access is access all Buffer_Container_Type;
   procedure Free is new Ada.Unchecked_Deallocation (Buffer_Container_Type,
                                                     Buffer_Container_Access);
   --  This function returns true if the two containers contains a buffer with
   --  the same name.
   package Buffer_Container_Vectors is new Ada.Containers.Vectors
     (Positive, Buffer_Container_Access, "=");
   --  This function returns true if a buffer container contains a buffer named
   --  Name.
   function Buffer_Exist
     (Buffer_Container_Vector : Buffer_Container_Vectors.Vector;
      Name : String) return Boolean;
   function Get_Buffer_Container
     (Buffer_Container_Vector : Buffer_Container_Vectors.Vector;
      Name : String) return Buffer_Container_Access;
   pragma Inline (Get_Buffer_Container);

   protected Memory_Handler is
     --  This function returns, as an XML string, the state of all the buffers
     --  Posix Manager Handler contains.
     function Get_State return String;
     --  This function returns, as an XML string, the state of a given buffer.
     function Get_Buffer_State (Name : String) return String;
     function Get_Buffer_Size (Name : String) return Buffer_Size_Type;
     --  This function create the shared buffer. It needs a name, a size (in
     --  bytes) and a depth. The default depth is 3 and the default size is
     --  65536 bytes.
     procedure Create_Buffer (Name : String;
                              Buffer_Container : out Buffer_Container_Access;
                              Size : Common_Types.Buffer_Size_Type := 65_536;
                              Depth : Positive := 3);
     procedure Reset_Buffer (Name : String);
     procedure Delete_Buffer (Name : String);
     --  This procedure enable/disable a policy for the buffer referenced by
     --  Name.
     procedure Set_Policy (Name : String;
                           Policy : Common_Types.Policy_Type;
                           Value : Boolean);
     --  This procedure enable/disable a mode for the buffer referenced by
     --  Name.
     procedure Set_Mode (Name : String;
                         Mode : Common_Types.Mode_Type;
                         Value : Boolean);
     --  This function returns an indefinite vector of buffer name.
     function Get_Buffer_Names return Common_Types.String_Vectors.Vector;
     function Buffer_Exist (Name : String) return Boolean;
   private
      Buffers_Vector : Buffer_Container_Vectors.Vector;
   end Memory_Handler;

   Shell_Error : exception;

end Buffers.Shell;
