with Ada.Strings.Unbounded;

with Common_Types;

package DCOD.Pmh.Real_Handler is

   task Memory_Handler_Task is
     entry Provide_Buffer (Name : String;
                           Size : Common_Types.Buffer_Size_Type := 65_536;
                           Depth : Positive := 3);
     --  This entry prepare the host part of a shared buffer named Name.
     --  It calls create if the buffer doesn't exists
     entry Request_Buffer (Name : String);
     entry Reset_Buffer (Name : String);
     entry Delete_Buffer (Name : String);
     --  This entry enable/disable a policy for the buffer referenced by
     --  Name.
     entry Set_Policy (Name : String;
                           Policy : Common_Types.Policy_Type;
                           Value : Boolean);
     --  This entry enable/disable a mode for the buffer referenced by
     --  Name.
     entry Set_Mode (Name : String;
                         Mode : Common_Types.Mode_Type;
                         Value : Boolean);
     --  This entry is use to asynchronously trig some event in any object
     --  of the DCOD framework.
     entry Trig_Event (Event : String);
   end Memory_Handler_Task;

   function Get_State return String;
   function Get_Buffer_State (Name : String) return String;

end DCOD.Pmh.Real_Handler;
