with Ada.Exceptions;
with Ada.Text_IO;

with GNAT.Traceback.Symbolic;

with AWS.Server;
with AWS.Config.Set;

with SOAP.Dispatchers.Callback;

with Global_Options;
with DCOD.Pmh.Http;
with DCOD.Pmh.Soap_Cb;

with Broadcast_Stuff;
with Broadcast_Buffers;
with Pmh_Directory;

procedure Posix_Memory_Handler_Stand_Alone is
   AWS_Server : AWS.Server.HTTP;
   Config : AWS.Config.Object;
   Name : constant String := Global_Options.Get_Name;
begin
   AWS.Config.Set.Server_Name (Config, "Posix Memory Handler");
   AWS.Config.Set.Reuse_Address (Config, True);
   Broadcast_Stuff.Set_Name (Pmh_Directory.An_Entry, Name);
   Pmh_Directory.Start;
   if Global_Options.Get_Port = 0 then
      --  Default AWS port is 5678
      AWS.Config.Set.Server_Port (Config, 5678);
      Broadcast_Stuff.Set_Port (Pmh_Directory.An_Entry, 5678);
   else
      AWS.Config.Set.Server_Port (Config, Global_Options.Get_Port);
      Broadcast_Stuff.Set_Port (Pmh_Directory.An_Entry, Global_Options.Get_Port);
   end if;
   AWS.Server.Start (AWS_Server,
                     Dispatcher => SOAP.Dispatchers.Callback.Create
                       (DCOD.Pmh.Http.Callback'Access,
                        DCOD.Pmh.Soap_Cb.Callback'Access),
                     Config => Config);
   AWS.Server.Wait (AWS.Server.Forever);
   Pmh_Directory.Stop;
   Broadcast_Buffers.Close;
   Ada.Text_IO.Put_Line ("out of Posix_Memory_Handler_Run : " & Name);
exception
   when E : others =>
      Pmh_Directory.Stop;
      Broadcast_Buffers.Close;
      Ada.Text_IO.Put_Line ("   exception name : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("exception message : " & Ada.Exceptions.Exception_Message (E));
      Ada.Text_IO.Put_Line (GNAT.Traceback.Symbolic.Symbolic_Traceback (E));
end Posix_Memory_Handler_Stand_Alone;
