with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;

with Common_Types;
with Dcod_Pmh_Service.Client;

procedure Pmh_Soap_Shell is
   use Ada.Strings.Unbounded;
   type Action_Enum is (Create, Delete, Set_Mode, Set_Policy, Get);
   Options : constant String :=
     "number_buffer= action= buffer_name= pmh_port=" &
     " pmh_host= value= mode= policy= size=";
   Host_Name : Unbounded_String := To_Unbounded_String ("localhost");
   Pmh_Port : Positive := 5678;
   Buffer_Name : Unbounded_String := Null_Unbounded_String;
   Action : Action_Enum := Get;
   Number_Of_Buffer : Positive := 20;
   Value : Unbounded_String := Null_Unbounded_String;
   Mode : Common_Types.Mode_Type := Common_Types.No_Transmit;
   Policy : Common_Types.Policy_Type := Common_Types.Consume_On_Fly;
   Size : Integer := 65_536;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'a' =>
            Action := Action_Enum'Value (GNAT.Command_Line.Parameter);
         when 'b' =>
            Buffer_Name := To_Unbounded_String
              (GNAT.Command_Line.Parameter);
         when 'm' =>
            Mode := Common_Types.Mode_Type'Value (GNAT.Command_Line.Parameter);
         when 'n' =>
            Number_Of_Buffer := Positive'Value
              (GNAT.Command_Line.Parameter);
         when 'p' =>
            if GNAT.Command_Line.Full_Switch = "pmh_port" then
               Pmh_Port := Positive'Value (GNAT.Command_Line.Parameter);
            elsif GNAT.Command_Line.Full_Switch = "pmh_host" then
               Host_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
            elsif GNAT.Command_Line.Full_Switch = "policy" then
               Policy := Common_Types.Policy_Type'Value
                 (GNAT.Command_Line.Parameter);
            end if;
         when 's' =>
            Size := Integer'Value (GNAT.Command_Line.Parameter);
         when 'v' =>
            Value := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   declare
      End_Point : String := To_String ("http://" & Host_Name & Pmh_Port'Img);
   begin
      for I in End_Point'Range loop
         if End_Point (I) = ' ' then
            End_Point (I) := ':';
         end if;
      end loop;
      case Action is
         when Create =>
            if Buffer_Name = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("buffer name not set, can't create buffer");
               return;
            end if;
            Dcod_Pmh_Service.Client.Provide_Buffer
              (Name => To_String (Buffer_Name),
               Size => Size,
               Depth => Number_Of_Buffer,
               Endpoint => End_Point);
         when Delete =>
            if Buffer_Name = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("buffer name not set, can't delete buffer");
               return;
            end if;
            Dcod_Pmh_Service.Client.Delete_Buffer
              (Name => To_String (Buffer_Name),
               Endpoint => End_Point);
         when Set_Mode =>
            if Value = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("value name not set,can't change mode");
               return;
            end if;
            if Buffer_Name = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("buffer name not set, can't change mode");
               return;
            end if;
            declare
               Value_Boolean : constant Boolean :=
                 Boolean'Value (To_String (Value));
            begin
               Dcod_Pmh_Service.Client.Set_Mode
                 (Name => To_String (Buffer_Name),
                  Mode => Mode,
                  Value => Value_Boolean,
                  Endpoint => End_Point);
            end;
         when Set_Policy =>
            if Value = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("value name not set,can't change policy");
               return;
            end if;
            if Buffer_Name = Null_Unbounded_String then
               Ada.Text_IO.Put_Line
                 ("buffer name not set, can't change policy");
               return;
            end if;
            declare
               Value_Boolean : constant Boolean :=
                 Boolean'Value (To_String (Value));
            begin
               Dcod_Pmh_Service.Client.Set_Policy
                 (Name => To_String (Buffer_Name),
                  Policy => Policy,
                  Value => Value_Boolean,
                  Endpoint => End_Point);
            end;
         when Get =>
            if Buffer_Name = Null_Unbounded_String then
               Ada.Text_IO.Put_Line (Dcod_Pmh_Service.Client.Get_State
                                       (Endpoint => End_Point));
            else
               Ada.Text_IO.Put_Line (Dcod_Pmh_Service.Client.Get_Buffer_State
                                       (Name => To_String (Buffer_Name),
                                        Endpoint => End_Point));
            end if;
      end case;
   end;
end Pmh_Soap_Shell;
