with Ada.Strings.Unbounded;
with Ada.Text_IO;
with Interfaces;

with GNAT.Command_Line;

with Buffers.Shared.Consume;

with Dcod_Pmh_Service.Client;

procedure Soap_Empty_Buffer is
   Options : constant String := "buffer_name= pmh_port=";
   Consumption : Buffers.Shared.Consume.Consume_Couple_Type;
   Buffer : Buffers.Shared.Consume.Consume_Type renames Consumption.Consumer;
   use Ada.Strings.Unbounded;
   Buffer_Name : Unbounded_String := To_Unbounded_String ("green_buffer");
   Pmh_Port : Positive := 5678;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'b' =>
            Buffer_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Pmh_Port := Positive'Value (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   Buffer.Set_Name (To_String (Buffer_Name));
   Consumption.Message_Handling.Start (1.0);
   Ada.Text_IO.Put_Line ("empty buffer, step 1");
   loop
      declare
         Buffer_Handle : Buffers.Buffer_Handle_Type;
         use Buffers;
      begin
         Ada.Text_IO.Put_Line ("empty buffer, step 4");
         select
            Buffer.Get_Full_Buffer (Buffer_Handle);
         or
            delay 5.0;
            Ada.Text_IO.Put_Line ("no buffers available");
            exit;
         end select;
         declare
            Datas : array (1 .. 1024) of Interfaces.Unsigned_32;
            for Datas'Address use Get_Address (Buffer_Handle);
            for Datas'Size use 32768;
         begin
            Ada.Text_IO.Put_Line ("Datas (Datas'first) : " &
                                    Datas (Datas'First)'Img);
         end;
         Buffer.Release_Full_Buffer (Buffer_Handle);
      end;
   end loop;
   Ada.Text_IO.Put_Line ("empty buffer, step 5");
   Buffer.Delete;
   Ada.Text_IO.Put_Line ("empty buffer, step 6");
   declare
      End_Point : String := "http://localhost" & Pmh_Port'Img;
   begin
      for I in End_Point'Range loop
         if End_Point (I) = ' ' then
            End_Point (I) := ':';
         end if;
      end loop;
      Dcod_Pmh_Service.Client.Delete_Buffer (Name => To_String (Buffer_Name),
                                             Endpoint => End_Point);
   end;
   Ada.Text_IO.Put_Line ("empty buffer, step 7");
   Consumption.Message_Handling.Stop;
end Soap_Empty_Buffer;
