with Ada.Strings.Unbounded;
with Ada.Text_IO;
with Interfaces;

with GNAT.Command_Line;

with Buffers.Shared.Produce;

with Dcod_Pmh_Service.Client;

procedure Soap_Fill_Buffer is
   Options : constant String :=
     "buffer_name= delay= number_of_buffer= pmh_port=";
   Start_Delay : Duration := 0.0;
   Buffer_Size : constant := 32768;
   use Ada.Strings.Unbounded;
   Buffer_Name : Unbounded_String := To_Unbounded_String ("green_buffer");
   Production : Buffers.Shared.Produce.Produce_Couple_Type;
   Buffer : Buffers.Shared.Produce.Produce_Type renames Production.Producer;
   Request_Counter : Natural := 0;
   Number_Of_Buffer : Positive := 20;
   Pmh_Port : Positive := 5678;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'b' =>
            Buffer_Name := To_Unbounded_String
              (GNAT.Command_Line.Parameter);
         when 'd' =>
            Start_Delay := Duration'Value (GNAT.Command_Line.Parameter);
         when 'n' =>
            Number_Of_Buffer := Positive'Value
              (GNAT.Command_Line.Parameter);
         when 'p' =>
            Pmh_Port := Positive'Value (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   declare
      End_Point : String := "http://localhost" & Pmh_Port'Img;
   begin
      for I in End_Point'Range loop
         if End_Point (I) = ' ' then
            End_Point (I) := ':';
         end if;
      end loop;
      Dcod_Pmh_Service.Client.Provide_Buffer (Name => To_String (Buffer_Name),
                                              Size => Buffer_Size,
                                              Depth => Number_Of_Buffer,
                                              Endpoint => End_Point);
   end;
   Ada.Text_IO.Put_Line ("waiting" & Start_Delay'Img &
                           " seconds before filling");
   Buffer.Set_Name (To_String (Buffer_Name));
   Production.Message_Handling.Start (1.0);
   delay Start_Delay;
   select
      Buffer.Is_Initialised;
   or
      delay 3.0;
      Ada.Text_IO.Put_Line ("too long to initialise");
      Production.Message_Handling.Stop;
      Buffer.Delete;
      return;
   end select;
   loop
      declare
         Buffer_Handle : Buffers.Buffer_Handle_Type;
         use Buffers;
      begin
         Ada.Text_IO.Put_Line ("request memory :" & Request_Counter'Img);
         select
            Buffer.Get_Free_Buffer (Buffer_Handle);
         or
            delay 5.0;
            Ada.Text_IO.Put_Line ("too long to get free buffer");
            exit;
         end select;
         declare
            Datas : array (1 .. 1024) of Interfaces.Unsigned_32;
            for Datas'Address use Get_Address (Buffer_Handle);
            for Datas'Size use Buffer_Size;
         begin
            Ada.Text_IO.Put_Line ("fill memory");
            for I in Datas'Range loop
               Datas (I) := Interfaces.Unsigned_32 (I + Request_Counter);
            end loop;
            Set_Used_Bytes (Buffer_Handle, 4096);
            Ada.Text_IO.Put_Line ("before release memory");
            Buffer.Release_Free_Buffer (Buffer_Handle);
            Ada.Text_IO.Put_Line ("release memory");
            delay 1.0;
         end;
         Request_Counter := Request_Counter + 1;
         exit when Request_Counter = Number_Of_Buffer;
      end;
   end loop;
   Buffer.Delete;
   Production.Message_Handling.Stop;
end Soap_Fill_Buffer;
