with Ada.Text_IO;
with Ada.Exceptions;
with Interfaces;

with Buffers.Local;

procedure Test_Buffers is
   package Unsigned_32_Buffers is new
     Buffers.Generic_Buffers (Element_Type => Interfaces.Unsigned_32);
   task type Producer_Task
     (Buffer : Buffers.Local.Local_Buffer_Access) is
     entry Start;
     entry Stop;
   end Producer_Task;
   task type Consumer_Task
     (Buffer : Buffers.Local.Local_Buffer_Access) is
     entry Start;
     entry Stop;
   end Consumer_Task;
   task body Producer_Task is
   begin
      accept Start;
      loop
         Ada.Text_IO.Put_Line ("producer task waiting");
         delay 1.0;
         select
            accept Stop;
            exit;
         else
            declare
               use Unsigned_32_Buffers;
               Buffer_Handle : Buffers.Buffer_Handle_Type;
            begin
               Buffer.Get_Free_Buffer (Buffer_Handle);
               declare
                  type Data_Array is new
                    Element_Array
                    (1 .. To_Word_Count
                     (Buffers.Get_Available_Bytes (Buffer_Handle)));
                  Datas : Data_Array;
                  for Datas'Address use Buffers.Get_Address (Buffer_Handle);
               begin
                  for I in Datas'Range loop
                     Datas (I) := Interfaces.Unsigned_32 (I);
                  end loop;
                  Buffers.Set_Used_Bytes (Buffer_Handle,
                                          To_Bytes (Datas'Length));
               end;
               Buffer.Release_Free_Buffer (Buffer_Handle);
            exception
               when E : others =>
                  Ada.Text_IO.Put_Line ("exception : " &
                                        Ada.Exceptions.Exception_Name (E) &
                                        " message : " &
                                        Ada.Exceptions.Exception_Message (E));
            end;
         end select;
      end loop;
   end Producer_Task;
   task body Consumer_Task is
   begin
      accept Start;
      loop
         select
            accept Stop;
            exit;
         else
            declare
               use Unsigned_32_Buffers;
               Buffer_Handle : Buffers.Buffer_Handle_Type;
            begin
               select
                  Buffer.Get_Full_Buffer (Buffer_Handle);
                  declare
                     type Data_Array is new Element_Array
                       (1 .. To_Word_Count
                        (Buffers.Get_Used_Bytes (Buffer_Handle)));
                     Datas : Data_Array;
                     for Datas'Address use Buffers.Get_Address (Buffer_Handle);
                  begin
                     Ada.Text_IO.Put_Line ("data (data'first) : " &
                                           Datas (Datas'First)'Img);
                  end;
                  Buffer.Release_Full_Buffer (Buffer_Handle);
               or
                  delay 0.5;
                  Ada.Text_IO.Put_Line ("waiting for full buffer");
               end select;
            exception
               when E : others =>
                  Ada.Text_IO.Put_Line ("exception : " &
                                        Ada.Exceptions.Exception_Name (E) &
                                        " message : " &
                                        Ada.Exceptions.Exception_Message (E));
            end;
         end select;
      end loop;
   end Consumer_Task;
   A_Buffer : aliased Buffers.Local.Local_Buffer_Type;
   A_Producer : Producer_Task (A_Buffer'Unchecked_Access);
   A_Second_Producer : Producer_Task (A_Buffer'Unchecked_Access);
   A_Consumer : Consumer_Task (A_Buffer'Unchecked_Access);
begin
   A_Buffer.Initialise (10, Size => 4096);
   Ada.Text_IO.Put_Line (A_Buffer.Get_State);
   A_Producer.Start;
   A_Second_Producer.Start;
   delay 3.0;
   Ada.Text_IO.Put_Line (A_Buffer.Get_State);
   A_Consumer.Start;
   A_Producer.Stop;
   delay 3.0;
   A_Second_Producer.Stop;
   delay 3.0;
   A_Consumer.Stop;
   Ada.Text_IO.Put_Line (A_Buffer.Get_State);
   A_Buffer.Delete;
   Ada.Text_IO.Put_Line (A_Buffer.Get_State);
end Test_Buffers;
