with Ada.Exceptions;
with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;

with Buffers.Mailboxes.Strings;
with Passive_Directory;

procedure Test_Mailbox_Emitter is
   Options : constant String := "message= pmh= name=";
   use Ada.Strings.Unbounded;
   Message : Unbounded_String := To_Unbounded_String ("elle souffle");
   Name : Unbounded_String := To_Unbounded_String ("local");
   Pmh : Unbounded_String := To_Unbounded_String ("test_mailbox");
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'm' =>
            Message := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'n' =>
            Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Pmh := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;

   Ada.Text_IO.Put_Line (To_String ("emission step 1 : " & Name & "/" & Message & "/" & Pmh));
   Passive_Directory.Start;

   declare
      Mailbox : Buffers.Mailboxes.Mailbox_Send_Type;
   begin
      Buffers.Mailboxes.Open_Send (Mailbox => Mailbox,
                                   Name => To_String (Name),
                                   Pmh => To_String (Pmh));
      Ada.Text_IO.Put_Line ("mailbox open");
      Buffers.Mailboxes.Strings.Send_String (Mailbox, To_String (Message));
      Ada.Text_IO.Put_Line ("message sent");
      Passive_Directory.Stop;
      delay 2.0;
      Ada.Text_IO.Put ("closing mailmox...");
      Buffers.Mailboxes.Close (Mailbox);
      Ada.Text_IO.Put_Line ("done");
   end;
exception
   when E : others =>
      Passive_Directory.Stop;
      Ada.Text_IO.Put_Line ("end on unexpected exception : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("                    message : " & Ada.Exceptions.Exception_Message (E));
end Test_Mailbox_Emitter;
