with Ada.Exceptions;
with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;

with Buffers.Mailboxes.Strings;
with Passive_Directory;

procedure Test_Mailbox_Receiver is
   Options : constant String := "pmh= name=";
   use Ada.Strings.Unbounded;
   Name : Unbounded_String := To_Unbounded_String ("local");
   Pmh : Unbounded_String := To_Unbounded_String ("test_mailbox");
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'n' =>
            Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Pmh := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;

   Ada.Text_IO.Put_Line (To_String ("reception step 1 : " & Name & "/" & Pmh));
   Passive_Directory.Start;

   declare
      Mailbox : Buffers.Mailboxes.Mailbox_Receive_Type;
   begin
      Buffers.Mailboxes.Open_Receive (Mailbox => Mailbox,
                                      Name => To_String (Name),
                                      Pmh => To_String (Pmh));
      Ada.Text_IO.Put_Line ("mailbox open");
      declare
         Message : constant String :=
           Buffers.Mailboxes.Strings.Receive_String (Mailbox);
      begin
         Ada.Text_IO.Put_Line ("message : " & Message);
         delay 2.0;
         Ada.Text_IO.Put ("closing mailmox...");
         Buffers.Mailboxes.Close (Mailbox);
         Ada.Text_IO.Put_Line ("done");
      end;
   end;
   Passive_Directory.Stop;
exception
   when E : others =>
      Passive_Directory.Stop;
      Ada.Text_IO.Put_Line ("end on unexpected exception : " & Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("                    message : " & Ada.Exceptions.Exception_Message (E));
end Test_Mailbox_Receiver;
