with Interfaces;
with Ada.Text_IO;
with Ada.Exceptions;

with Buffers.Shared.Consume;
with GNAT.Command_Line;

procedure Test_Shared_Buffers_Cons is
   Consumption : Buffers.Shared.Consume.Consume_Couple_Type;
   Buffer : Buffers.Shared.Consume.Consume_Type renames Consumption.Consumer;
   Long_Delay : Boolean := False;
   Options : constant String := "name= long_delay= on_fly";
   No_Name : Boolean := True;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'n' =>
            Buffer.Set_Name (GNAT.Command_Line.Parameter);
            No_Name := False;
         when 'l' =>
            Long_Delay := Boolean'Value (GNAT.Command_Line.Parameter);
         when 'o' =>
            Buffer.Set_On_Fly_Flag;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if No_Name then
      Buffer.Set_Name ("green_buffer");
   end if;
   Consumption.Message_Handling.Start (1.0);
   loop
      Ada.Text_IO.Put_Line ("in main loop : step 1");
      declare
         Buffer_Handle : Buffers.Buffer_Handle_Type;
         use Buffers;
      begin
         select
            Buffer.Get_Full_Buffer (Buffer_Handle);
            Ada.Text_IO.Put_Line ("in main loop : step 2, get buffer");
         or
            delay 5.0;
            Ada.Text_IO.Put_Line ("no buffers available");
            exit;
         end select;
         declare
            Datas : array (1 .. 1024) of Interfaces.Unsigned_32;
            for Datas'Address use Get_Address (Buffer_Handle);
            for Datas'Size use 32768;
         begin
            Ada.Text_IO.Put_Line ("Datas (Datas'first) : " &
                                  Datas (Datas'First)'Img);
         end;
         Buffer.Release_Full_Buffer (Buffer_Handle);
         Ada.Text_IO.Put_Line ("in main loop : step 3 buffer released");
         if Long_Delay then
            delay 2.0;
         end if;
      end;
   end loop;
   Ada.Text_IO.Put_Line ("before messages handling stop");
   Consumption.Message_Handling.Stop;
   Ada.Text_IO.Put_Line ("before delete buffer");
   Buffer.Delete;
exception
   when E : others =>
      Ada.Text_IO.Put_Line ("Exception Received : " &
                              Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("Exception message : " &
                              Ada.Exceptions.Exception_Message (E));
      Buffer.Delete;
end Test_Shared_Buffers_Cons;
