with Ada.Text_IO;

with GNAT.Command_Line;

with Common_Types;
with Buffers.Shared.Host;
with Utiles_Task;

procedure Test_Shared_Buffers_Host is
   Messages_Hangling : aliased Buffers.Shared.Host.Message_Handling_Task;
   Buffer : aliased Buffers.Shared.Host.Host_Type
     (Messages_Hangling'Unchecked_Access);
   Loop_Counter : Natural := 0;
   Options : constant String := "name= policy= limit=";
   No_Name : Boolean := True;
   Policy : Common_Types.Policy_Type;
   use type Common_Types.Policy_Type;
   Limit : Natural := 60;
   A_Task_Communication : aliased Utiles_Task.Task_Communication;
   A_Terminate_Task : Utiles_Task.Terminate_Task (A_Task_Communication'Access);
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'l' =>
            Limit := Natural'Value (GNAT.Command_Line.Parameter);
         when 'n' =>
            Buffer.Set_Name (GNAT.Command_Line.Parameter);
            No_Name := False;
         when 'p' =>
            begin
               Policy := Common_Types.Policy_Type'Value
                 (GNAT.Command_Line.Parameter);
               Buffer.Set_Policy (Policy, True);
               if Policy = Common_Types.Consume_On_Fly then
                  Buffer.Set_On_Fly_Limit (1);
               end if;
            exception
               when Buffers.Not_Supported_Policy =>
                  Ada.Text_IO.Put_Line (GNAT.Command_Line.Parameter &
                                          " policy not supported");
                  for I in Common_Types.Policy_Type loop
                     Ada.Text_IO.Put_Line
                       ("    " & I'Img &
                          " supported : " &
                          Buffer.Get_Available_Policies (I)'Img);
                  end loop;
                  return;
               when others =>
                  Ada.Text_IO.Put_Line ("available policies :");
                  for I in Common_Types.Policy_Type loop
                     Ada.Text_IO.Put_Line
                       ("    " & I'Img &
                          " supported : " &
                          Buffer.Get_Available_Policies (I)'Img);
                  end loop;
                  return;
            end;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if No_Name then
      Buffer.Set_Name ("green_buffer");
   end if;
   Buffer.Initialise (Buffer_Number => 16,
                      Size => 65536);
   Messages_Hangling.Start (Buffer'Unchecked_Access, 1.0);
   if Limit = 0 then
      loop
         select
            A_Task_Communication.Can_Stop;
            exit;
         or
            delay 1.0;
            Ada.Text_IO.Put_Line (Buffer.Get_State);
         end select;
      end loop;
   else
      loop
         Ada.Text_IO.Put_Line (Buffer.Get_State);
         Loop_Counter := Loop_Counter + 1;
         delay 1.0;
         exit when Loop_Counter = Limit;
      end loop;
   end if;
   Messages_Hangling.Stop;
   loop
      select
         Buffer.Delete;
         exit;
      or
         delay 1.0;
         Ada.Text_IO.Put_Line ("buffer seems long to delete");
      end select;
   end loop;
end Test_Shared_Buffers_Host;
