with Interfaces;
with Ada.Exceptions;
with Ada.Text_IO;

with Buffers.Shared.Produce;
with GNAT.Command_Line;

procedure Test_Shared_Buffers_Prod is
   Production : Buffers.Shared.Produce.Produce_Couple_Type;
   Buffer : Buffers.Shared.Produce.Produce_Type renames Production.Producer;
   Request_Counter : Natural := 0;
   Options : constant String := "name=";
   No_Name : Boolean := True;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'n' =>
            Buffer.Set_Name (GNAT.Command_Line.Parameter);
            No_Name := False;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if No_Name then
      Buffer.Set_Name ("green_buffer");
   end if;
   Production.Message_Handling.Start (1.0);
   select
      Buffer.Is_Initialised;
   or
      delay 3.0;
      Ada.Text_IO.Put_Line ("too long to initialise");
      Production.Message_Handling.Stop;
      Buffer.Delete;
      return;
   end select;
   loop
      declare
         Buffer_Handle : Buffers.Buffer_Handle_Type;
         use Buffers;
      begin
         Ada.Text_IO.Put_Line ("request memory :" & Request_Counter'Img);
         select
            Buffer.Get_Free_Buffer (Buffer_Handle);
         or
            delay 10.0;
            Ada.Text_IO.Put_Line ("too long to get buffer");
            exit;
         end select;
         declare
            Datas : array (1 .. 1024) of Interfaces.Unsigned_32;
            for Datas'Address use Get_Address (Buffer_Handle);
            for Datas'Size use 32768;
         begin
            Ada.Text_IO.Put_Line ("fill memory");
            for I in Datas'Range loop
               Datas (I) := Interfaces.Unsigned_32 (I + Request_Counter);
            end loop;
            Set_Used_Bytes (Buffer_Handle, 4096);
            Ada.Text_IO.Put_Line ("before release memory");
            Buffer.Release_Free_Buffer (Buffer_Handle);
            Ada.Text_IO.Put_Line ("release memory");
            delay 1.0;
         end;
         Request_Counter := Request_Counter + 1;
         exit when Request_Counter = 20;
      end;
   end loop;
   Ada.Text_IO.Put_Line ("before stop task");
   Production.Message_Handling.Stop;
   Ada.Text_IO.Put_Line ("after stop task");
   Buffer.Delete;
   Ada.Text_IO.Put_Line ("producer end");
exception
   when E : others =>
      Buffer.Delete;
      Ada.Text_IO.Put_Line ("Exception Received : " &
                            Ada.Exceptions.Exception_Name (E));
      Ada.Text_IO.Put_Line ("Exception message : " &
                            Ada.Exceptions.Exception_Message (E));
end Test_Shared_Buffers_Prod;
