with Common_Types;

package DCOD.Interfaces.Data_Flow_Agent is
   pragma Remote_Types;

   type Data_Flow_Agent_Interface is synchronized interface
     and DCOD_Object_Interface;
   --  This function returns all the arguments of the agent.
   --  If Xml is true the string is a XML one.
   function Arguments (Object : Data_Flow_Agent_Interface;
                       Xml : Boolean := True)
                      return String is abstract;
   --  This function returns the value of one parameter as an XML string.
   function Get_Xml (Object : Data_Flow_Agent_Interface;
                     Parameter : String) return String is abstract;
   --  This function returns the state of the agent.
   function Get_State (Object : Data_Flow_Agent_Interface)
                      return Common_Types.Acquisition_State_Type is abstract;
   --  This function returns an argument as a string.
   function Get_Image (Object : Data_Flow_Agent_Interface;
                       Parameter_Name : String) return String is abstract;
   --  This procedure sets an argument with the value Value. Conversion from
   --  string to final type is taken into account by the agent itself.
   procedure Set (Object : in out Data_Flow_Agent_Interface;
                  Parameter_Name : String;
                  Value : String) is abstract;
   --  This procedure allow a data flow agent to move inside the embedded
   --  state machine each instance owns.
   procedure Change_State (Object : in out Data_Flow_Agent_Interface;
                           Order : Common_Types.Action) is abstract;

   type Agent_Access is access all Data_Flow_Agent_Interface'Class;
   type Agent_Array is array (Positive range <>) of Agent_Access;

end DCOD.Interfaces.Data_Flow_Agent;
