with Common_Types;
with DCOD.Interfaces.Data_Flow_Agent;

package DCOD.Interfaces.Data_Flow_Handler is
   pragma Remote_Types;

   type Data_Flow_Handler_Interface is synchronized interface
     and DCOD_Object_Interface;
   --  This function returns the acquisition state.
   function Get_State (Object : Data_Flow_Handler_Interface)
                      return Common_Types.Acquisition_State_Type is abstract;
   --  This function returns all the arguments of the data flow handler.
   --  If Xml is true the string is a XML one.
   function Arguments (Object : Data_Flow_Handler_Interface;
                       Xml : Boolean := True)
                      return String is abstract;
   --  This function returns the value of one parameter as an XML string.
   function Get_Xml (Object : Data_Flow_Handler_Interface;
                     Parameter : String) return String is abstract;
   --  This function returns the fat pointer to an agent named Name.
   function Get_Agent (Object : Data_Flow_Handler_Interface;
                       Name : String)
                      return Data_Flow_Agent.Agent_Access is abstract;
   --  This procedure allow a data flow handler to move inside the embedded
   --  state machine each instance owns.
   procedure Change_State (Object : in out Data_Flow_Handler_Interface;
                           Order : Common_Types.Action) is abstract;
   --  This procedure puts a data flow handler in error state.
   procedure Put_In_Error (Object : in out Data_Flow_Handler_Interface)
      is abstract;
   --  This procedure sets an argument with the value Value. Conversion from
   --  string to final type is taken into account by the handler itself.
   procedure Set (Object : in out Data_Flow_Handler_Interface;
                  Parameter_Name : String;
                  Value : String) is abstract;
   --  This function returns an argument as a string.
   function Get_Image (Object : Data_Flow_Handler_Interface;
                       Parameter_Name : String) return String is abstract;
   --  This procedure is use by data flow agent to register they fat pointer in
   --  the data flow handler they belong to.
   procedure Register
     (Object : in out Data_Flow_Handler_Interface;
      Reg_Name : String;
      Pid : Integer;
      Pointer : Data_Flow_Agent.Agent_Access) is abstract;
   --  This function is used by an actor to retrieve it's configuration
   --  information from the data flow handler it belong to.
   function Get_Informations (Object : Data_Flow_Handler_Interface;
                              Actor_Name : String) return String is abstract;

   type Data_Flow_Handler_Access is
     access all Data_Flow_Handler_Interface'Class;
   type Data_Flow_Handler_Array is
     array (Positive range <>) of Data_Flow_Handler_Access;

end DCOD.Interfaces.Data_Flow_Handler;
