package DCOD.Interfaces is

   pragma Preelaborate;
   pragma Remote_Types;

   type DCOD_Object_Interface is limited interface;
   --  This procedure waits for the end of the object.
   procedure Wait_For_End (Object : in out DCOD_Object_Interface) is abstract;
   --  This procedure informs the object that it should not provide any
   --  services anymore.
   procedure Go_To_End (Object : in out DCOD_Object_Interface) is abstract;
   --  This procedure is used to set the name of the DCOD object.
   procedure Set_Name (Object : in out DCOD_Object_Interface;
                       Name : String) is abstract;
   --  This procedure is used to retrieve the name of the DCOD object.
   function Get_Name (Object : DCOD_Object_Interface)
                     return String is abstract;
   --  This procedure is used to know if the process that host the object
   --  provides or not a embedded web server.
   function Has_Http_Server (Object : DCOD_Object_Interface)
                            return Boolean is abstract;
   --  This procedure returns the http URL needed to contact the embedded
   --  web server if Has_Http_Server returns True.
   function Server_Url (Object : DCOD_Object_Interface)
                       return String is abstract;
   --  This function returns the host name where the object is running.
   function Get_Host_Name (Object : DCOD_Object_Interface)
                          return String is abstract;
   --  This procedure is use to asynchronously trig some event in any object of
   --  the DCOD framework.
   procedure Trig_Event (Object : in out DCOD_Object_Interface;
                         Event : String) is abstract;
   type DCOD_Object_Access is access all DCOD_Object_Interface'Class;
   type DCOD_Object_Array is array (Positive range <>) of DCOD_Object_Access;

   Object_State_Not_Conform : exception;
   Object_Misused : exception;

end DCOD.Interfaces;
