with Ada.Containers.Vectors;

package body DCOD.Services is

   package Object_Vectors_Pkg is new Ada.Containers.Vectors
     (Positive, Interfaces.DCOD_Object_Access, Interfaces."=");
   package Dfh_Vectors_Pkg is new Ada.Containers.Vectors
     (Positive,
      Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access,
      Interfaces.Data_Flow_Handler."=");

   protected Object_Lists is
      --  This procedure add a fat pointer of DCOD_Object_Access type.
      procedure Append (Object : Interfaces.DCOD_Object_Access);
      --  This procedure add a fat pointer of Data_Flow_Handler_Access type.
      procedure Append
        (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access);
      --  This procedure remove a fat pointer of DCOD_Object_Access type.
      procedure Remove (Object : Interfaces.DCOD_Object_Access);
      --  This procedure remove a fat pointer of Data_Flow_Handler_Access type.
      procedure Remove
        (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access);
      --  This function returns all the Dfh suscribed as an array.
      function Get_Dfh return
        Interfaces.Data_Flow_Handler.Data_Flow_Handler_Array;
      --  This function returns the Dfh pointed by Name.
      function Get_Dfh (Name : String) return
        Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access;
      --  This function returns all the objects suscribed as an array.
      function Loc_Get_Objects return Interfaces.DCOD_Object_Array;
   private
      Objects : Object_Vectors_Pkg.Vector;
      Dfhs : Dfh_Vectors_Pkg.Vector;
   end Object_Lists;

   protected body Object_Lists is
      procedure Append
        (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access) is
      begin
         Dfhs.Append (Object);
      end Append;
      procedure Append (Object : Interfaces.DCOD_Object_Access) is
      begin
         Objects.Append (Object);
      end Append;
      procedure Remove
        (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access) is
      begin
         Dfhs.Delete (Dfhs.Find_Index (Object));
      end Remove;
      procedure Remove (Object : Interfaces.DCOD_Object_Access) is
      begin
         Objects.Delete (Objects.Find_Index (Object));
      end Remove;

      function Get_Dfh return
        Interfaces.Data_Flow_Handler.Data_Flow_Handler_Array is
      begin
         if Dfhs.Is_Empty then
            raise Empty_Vector with
              "No Data Flow Handlers are already suscribed";
         end if;
         declare
            Dfh_To_Return :
              Interfaces.Data_Flow_Handler.Data_Flow_Handler_Array
              (1 .. Positive (Dfhs.Length)) := (others => null);
            Index : Positive := 1;
         begin
            for Dfh of Dfhs loop
               Dfh_To_Return (Index) := Dfh;
               Index := Index + 1;
            end loop;
            return Dfh_To_Return;
         end;
      end Get_Dfh;

      function Get_Dfh (Name : String) return
        Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access is
      begin
         if Dfhs.Is_Empty then
            raise Empty_Vector with
              "No Data Flow Handlers are already suscribed";
         end if;
         for Dfh of Dfhs loop
            if Dfh.Get_Name = Name then
               return Dfh;
            end if;
         end loop;
         raise Object_Not_Found with
           Name & " isn't in the data flow handler list";
      end Get_Dfh;

      function Loc_Get_Objects return Interfaces.DCOD_Object_Array is
      begin
         if Objects.Is_Empty and
           Dfhs.Is_Empty
         then
            raise Empty_Vector with
              "No Objects are already suscribed";
         end if;
         declare
            use type Ada.Containers.Count_Type;
            Objects_To_Return : Interfaces.DCOD_Object_Array
              (1 .. Positive (Objects.Length +
                                Dfhs.Length)) :=
              (others => null);
            Index : Positive := 1;
         begin
            for Object of Objects loop
               Objects_To_Return (Index) := Object;
               Index := Index + 1;
            end loop;
            for Dfh of Dfhs loop
               Objects_To_Return (Index) :=
                 Interfaces.DCOD_Object_Access (Dfh);
               Index := Index + 1;
            end loop;
            return Objects_To_Return;
         end;
      end Loc_Get_Objects;
   end Object_Lists;

   procedure Subscribe_Data_Flow_Handler
     (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access) is
   begin
      Object_Lists.Append (Object);
   end Subscribe_Data_Flow_Handler;

   procedure Unsubscribe_Data_Flow_Handler
     (Object : Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access) is
   begin
      Object_Lists.Remove (Object);
   end Unsubscribe_Data_Flow_Handler;

   function Get_Data_Flow_Handlers
     return Interfaces.Data_Flow_Handler.Data_Flow_Handler_Array is
   begin
      return Object_Lists.Get_Dfh;
   end Get_Data_Flow_Handlers;

   function Get_Data_Flow_Handler (Name : String) return
     Interfaces.Data_Flow_Handler.Data_Flow_Handler_Access is
   begin
      return Object_Lists.Get_Dfh (Name);
   end Get_Data_Flow_Handler;

   procedure Subscribe_Object (Object : Interfaces.DCOD_Object_Access) is
   begin
      Object_Lists.Append (Object);
   end Subscribe_Object;

   procedure Unsubscribe_Object (Object : Interfaces.DCOD_Object_Access) is
   begin
      Object_Lists.Remove (Object);
   end Unsubscribe_Object;

   function Get_Objects return Interfaces.DCOD_Object_Array is
   begin
      return Object_Lists.Loc_Get_Objects;
   end Get_Objects;

   function Is_This_The_End return Boolean is
   begin
      return False;
   end Is_This_The_End;

   procedure End_Service (Force : Boolean) is
   begin
      null;
   end End_Service;

   Local_Event_Receiver : Events_Receivers.Events_Receivers_Class_Access :=
     null;
   pragma Atomic (Local_Event_Receiver);

   function Get_Event_Receiver
     return Events_Receivers.Events_Receivers_Class_Access is
      use type Events_Receivers.Events_Receivers_Class_Access;
   begin
      if Local_Event_Receiver = null then
         raise No_Event_Receiver with "event receiver pointer is null";
      end if;
      return Local_Event_Receiver;
   end Get_Event_Receiver;

   procedure Suscribe_Event_Receiver
     (Events_Receiver : Events_Receivers.Events_Receivers_Class_Access) is
   begin
      Local_Event_Receiver := Events_Receiver;
   end Suscribe_Event_Receiver;

end DCOD.Services;
