with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;
with GNAT.Sockets;

with AWS.Config.Set;
with AWS.Server;
with SOAP.Dispatchers.Callback;

with Dcod_Process_Launcher;
with Dcod_Process_Launcher_Service.CB;
with Dcod_Launcher_Http;
with Broadcast_Stuff;

procedure Dcod_Launcher is
   AWS_Server : AWS.Server.HTTP;
   Config : AWS.Config.Object;
   Options : constant String := "directory= port= name= debug=";
   Port : Natural := 0;
   use Ada.Strings.Unbounded;
   Name : Unbounded_String;
   Debug : Boolean := False;
   Launcher_Entry : aliased Broadcast_Stuff.Process_Entry_Type (Broadcast_Stuff.Launcher);
   Directory : aliased Broadcast_Stuff.Process_Directory.Directory_Type (Launcher_Entry'Access);
   Listener : Broadcast_Stuff.Broadcast_Process.Listener_Task;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'd' =>
            if GNAT.Command_Line.Full_Switch = "debug" then
               Debug := Boolean'Value (GNAT.Command_Line.Parameter);
            elsif GNAT.Command_Line.Full_Switch = "directory" then
               Dcod_Process_Launcher.Append_Directory
                 (GNAT.Command_Line.Parameter);
            else
               raise Program_Error;
            end if;
         when 'n' =>
            Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'p' =>
            Port := Positive'Value (GNAT.Command_Line.Parameter);
         when others =>
            raise Program_Error;
      end case;
   end loop;
   if Name = Null_Unbounded_String then
      Ada.Text_IO.Put_Line ("name option is mendatory");
      return;
   end if;
   if Port = 0 then
      Port := 9876;
   end if;
   Dcod_Process_Launcher.Set_Debug (Debug);
   AWS.Config.Set.Server_Port (Config, Port);
   Broadcast_Stuff.Set_Port (Launcher_Entry, Port);
   Broadcast_Stuff.Set_Name (Launcher_Entry, To_String (Name));
   Broadcast_Stuff.Set_Host (Launcher_Entry, GNAT.Sockets.Host_Name);
   Directory.Connect;
   Listener.Start (Directory'Unchecked_Access);
   AWS.Config.Set.Reuse_Address (Config, True);
   AWS.Server.Start (AWS_Server,
                     Dispatcher => SOAP.Dispatchers.Callback.Create
                       (Dcod_Launcher_Http.Callback'Access,
                        Dcod_Process_Launcher_Service.CB.SOAP_CB'Access),
                     Config => Config);
   AWS.Server.Wait;
end Dcod_Launcher;
