with Common_Types.Strings;

package Dcod_Process_Launcher is

   --  This procedure will launch a process accordingly to the executable name
   --  pointed by the variable Process_Name along with the associated
   --  parameters.
   procedure Launch (Process_Name : String;
                     Parameters : Common_Types.Strings.String_Couple_Array);

   --  This function returns the list of all the available processes in the
   --  directory pointed by the varaible Directory_Name. If Directory_Name
   --  contains #ALL_DIRECTORIES# then all the available processes will be
   --  returned.
   function Get_Executable_List (Directory_Name : String)
                                return Common_Types.Strings.String_Array;

   --  This functions returns the list of the process launched by this package
   --  associated with their status as a string.
   function Get_Running_List return Common_Types.Strings.String_Couple_Array;

   --  This function returns the properties of a given executable.
   function Get_Properties (Process_Name : String)
                           return Common_Types.Strings.String_Couple_Array;

   --  This procedure remove a listed process from the list. Ideally it
   --  shouldn't be called by the process itself. The kill option should be
   --  used to inforce cleaning.
   procedure Remove (Process_Name : String;
                     Parameters : Common_Types.Strings.String_Couple_Array;
                     Kill : Boolean);

   --  This procedure remove a listed process from the list. Ideally it
   --  shouldn't be called by the process itself. The kill option should be
   --  used to inforce cleaning. Here the process is selected by name plus Pid.
   procedure Remove_By_Pid (Process_Name : String;
                            Pid : Integer;
                            Kill : Boolean);

   --  This procedure isn't listed in the soap interface and is used to append
   --  directories where to find executables.
   procedure Append_Directory (Directory_Name : String);

   procedure Set_Debug (Debug : Boolean);

   API_Misused : exception;
   Executable_Not_Found : exception;
   Bad_Process_Name : exception;

end Dcod_Process_Launcher;
