with Ada.Strings.Unbounded;
with Ada.Text_IO;

with GNAT.Command_Line;

with Dcod_Process_Launcher_Service.Client;
with Dcod_Launcher_Tools;
with Common_Types.Strings;

procedure Dcod_Launcher_Shell is
   Options : constant String :=
     "arguments= command= directory= end_point= help kill= process_name= pid=";
   type Command_Type is (Help, Launch, List, Remove, Running);
   Command : Command_Type := Help;
   use Ada.Strings.Unbounded;
   End_Point : Unbounded_String :=
     To_Unbounded_String ("http://localhost:9876");
   Arguments : Unbounded_String := Null_Unbounded_String;
   Process_Name : Unbounded_String := Null_Unbounded_String;
   Directory_Name : Unbounded_String := To_Unbounded_String ("#ALL_DIRECTORIES#");
   Kill : Boolean := True;
   Pid : Natural := 0;
begin
   loop
      case GNAT.Command_Line.Getopt (Options) is
         when ASCII.NUL =>
            exit;
         when 'a' =>
            Arguments := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'c' =>
            Command := Command_Type'Value (GNAT.Command_Line.Parameter);
         when 'd' =>
            Directory_Name :=
              To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'e' =>
            End_Point := To_Unbounded_String (GNAT.Command_Line.Parameter);
         when 'h' =>
            Command := Help;
         when 'k' =>
            Kill := Boolean'Value (GNAT.Command_Line.Parameter);
         when 'p' =>
            if GNAT.Command_Line.Full_Switch = "process_name" then
               Process_Name := To_Unbounded_String (GNAT.Command_Line.Parameter);
            elsif GNAT.Command_Line.Full_Switch = "pid" then
               Pid := Natural'Value (GNAT.Command_Line.Parameter);
            else
               raise Program_Error with "Unexpected full switch " & GNAT.Command_Line.Full_Switch;
            end if;
         when others =>
            raise Program_Error;
      end case;
   end loop;
   case Command is
      when Help =>
         Ada.Text_IO.Put_Line ("available commands :");
         for I in Command_Type loop
            Ada.Text_IO.Put_Line ("   " & I'Img);
         end loop;
         Ada.Text_IO.Put_Line ("options string :" & Options);
      when Launch =>
         if Process_Name = Null_Unbounded_String then
            Ada.Text_IO.Put_Line ("process_name option have to be set in order to use action LAUNCH");
            return;
         end if;
         if Arguments = Null_Unbounded_String then
            Dcod_Process_Launcher_Service.Client.Launch
              (Process_Name => To_String (Process_Name),
               Parameters => Common_Types.Strings.Null_String_Couple_Array,
               Endpoint => To_String (End_Point));
         else
            Dcod_Process_Launcher_Service.Client.Launch
              (Process_Name => To_String (Process_Name),
               Parameters => Dcod_Launcher_Tools.To_Parameters (To_String (Arguments)),
               Endpoint => To_String (End_Point));
         end if;
      when List =>
         declare
            Executables : constant Common_Types.Strings.String_Array :=
              Dcod_Process_Launcher_Service.Client.Get_Executable_List
              (Directory_Name => To_String (Directory_Name),
               Endpoint => To_String (End_Point));
         begin
            for I in Executables'Range loop
               Ada.Text_IO.Put_Line
                 (I'Img & " " & To_String (Executables (I)));
            end loop;
         end;
      when Remove =>
         if Process_Name = Null_Unbounded_String then
            Ada.Text_IO.Put_Line ("process_name option have to be set in order to use action LAUNCH");
            return;
         end if;
         if Arguments = Null_Unbounded_String and Pid = 0 then
            Dcod_Process_Launcher_Service.Client.Remove
              (Process_Name => To_String (Process_Name),
               Parameters => Common_Types.Strings.Null_String_Couple_Array,
               Kill => Kill,
               Endpoint => To_String (End_Point));
         elsif Pid /= 0 then
            Dcod_Process_Launcher_Service.Client.Remove_By_Pid (Process_Name => To_String (Process_Name),
                                                                Pid => Pid,
                                                                Kill => Kill,
                                                                Endpoint => To_String (End_Point));
         else
            Dcod_Process_Launcher_Service.Client.Remove
              (Process_Name => To_String (Process_Name),
               Parameters => Dcod_Launcher_Tools.To_Parameters (To_String (Arguments)),
               Kill => Kill,
               Endpoint => To_String (End_Point));
         end if;
      when Running =>
         declare
            Running_List : constant Common_Types.Strings.String_Couple_Array :=
              Dcod_Process_Launcher_Service.Client.Get_Running_List
              (Endpoint => To_String (End_Point));
         begin
            for I in Running_List'Range loop
               Ada.Text_IO.Put_Line
                 (I'Img & " " &
                    To_String (Running_List (I).Name &
                                 Running_List (I).Value));
            end loop;
         end;
   end case;
end Dcod_Launcher_Shell;
