with Ada.Strings.Fixed;
with Ada.Strings.Unbounded;

with GNAT.String_Split;

with Dcod_Process_Launcher_Service.Client;

package body Dcod_Launcher_Tools is

   -------------------------
   -- Test_Launcher_Exist --
   -------------------------

   function Test_Launcher_Exist (End_Point : String) return Boolean is
   begin
      declare
         Executables : constant Common_Types.Strings.String_Array :=
           Dcod_Process_Launcher_Service.Client.Get_Executable_List
           (Directory_Name => "#ALL_DIRECTORIES#",
            Endpoint => End_Point);
         pragma Warnings (Off, Executables);
      begin
         return True;
      end;
   exception
      when others =>
         return False;
   end Test_Launcher_Exist;

   --------------------------
   -- Executable_Available --
   --------------------------

   function Executable_Available (Name : String;
                                  End_Point : String) return Boolean is
   begin
      declare
         Executables : constant Common_Types.Strings.String_Array :=
           Dcod_Process_Launcher_Service.Client.Get_Executable_List
           (Directory_Name => "#ALL_DIRECTORIES#",
            Endpoint => End_Point);
         use type Ada.Strings.Unbounded.Unbounded_String;
      begin
         for I in Executables'Range loop
            if Executables (I) = Name then
               return True;
            end if;
         end loop;
      end;
      return False;
   exception
      when others =>
         return False;
   end Executable_Available;

   function To_Parameters (Huge_String : String) return Common_Types.Strings.String_Couple_Array is
      Slices : GNAT.String_Split.Slice_Set;
      use GNAT.String_Split;
   begin
      Create (Slices, Huge_String, ";");
      declare
         Arguments :  Common_Types.Strings.String_Couple_Array (1 .. Integer (Slice_Count (Slices)));
         use Ada.Strings.Unbounded;
      begin
         for I in Arguments'Range loop
            declare
               Working_Slice : constant String := Slice (Slices, Slice_Number (I));
               Space_Position : constant Natural := Ada.Strings.Fixed.Index (Working_Slice, " ");
            begin
               if Space_Position = 0 then
                  Arguments (I).Name := Null_Unbounded_String;
                  Arguments (I).Value := To_Unbounded_String (Working_Slice);
               else
                  Arguments (I).Name :=
                    To_Unbounded_String (Working_Slice (Working_Slice'First .. Space_Position - 1));
                  Arguments (I).Value :=
                    To_Unbounded_String (Working_Slice (Space_Position + 1 .. Working_Slice'Last));
               end if;
            end;
         end loop;
         return Arguments;
      end;
   end To_Parameters;

end Dcod_Launcher_Tools;
