with P42.Html_Utils;
with AWS.MIME;
with Ada.Strings;
with Ada.Strings.Unbounded;
with Ada.Text_IO;
use Ada.Text_IO;
with Broadcast_Stuff;
with Passive_Directory;
with P42.String_Vectors;
use P42.String_Vectors;
with Ada.Calendar;
use Ada.Calendar;
with Ada.Calendar.Formatting;
with Ada.Calendar.Time_Zones;
use Ada.Calendar.Time_Zones;
use Ada.Calendar.Formatting;
with Format_Page;

with DCOD.Environment;

package body Dcod_Monitor_CB is

   use Ada.Strings.Unbounded;
   function "<" (Left, Right : Broadcast_Stuff.Process_Entry_Type) return Boolean;
   function "<" (Left, Right : Broadcast_Stuff.Process_Entry_Type) return Boolean is
      use type Broadcast_Stuff.Process_Enum;

   begin

      if Left.Name = Right.Name then
         return Left.Kind < Right.Kind;
      end if;
      return Left.Name < Right.Name;
   end "<";
   package Entry_Sorting is new Broadcast_Stuff.Process_Directory.Directory_Entry_Vectors.Generic_Sorting ("<");

   type Mode_Enum is (Auto, Manual);
   Current_Mode : Mode_Enum := Auto;

   function Dcod_Mr_CB (Request : AWS.Status.Data) return AWS.Response.Data is

      Page : P42.Html_Utils.T_Page_Html;
      URI : constant String := AWS.Status.URI (Request);
      Now : constant Time := Clock;
      Form_Mode_String : constant String := AWS.Status.Parameter (Request, "mode");
      Form_Mode : constant Mode_Enum :=
        (if Form_Mode_String = "" then Current_Mode else Mode_Enum'Value (Form_Mode_String));

      use type Broadcast_Stuff.Process_Enum;

   begin

      Ada.Text_IO.Put_Line (Image (Date => Now, Time_Zone => 1 * 60));
      Ada.Text_IO.Put_Line ("URI : " & URI);

      if Form_Mode /= Current_Mode then

         Current_Mode := Form_Mode;

      end if;
      if URI = "/" then

         Passive_Directory.Directory.Clear_Entries;
         Passive_Directory.Directory.Set_Listen;
         Passive_Directory.Directory.Ask_For_All;

         delay 0.1;

         declare

            Entries : Broadcast_Stuff.Process_Directory.Directory_Entry_Vectors.Vector :=
              Passive_Directory.Directory.Get_Entries;
            Hosts : P42.String_Vectors.Vector;

         begin

            Entry_Sorting.Sort (Entries);

            case Current_Mode is

               when Auto =>
                  Page.Head (Title => "Test Web Ada",
                             CSS_URI => "/css",
                             Refresh => 5.0);
               when Manual =>
                  Page.Head (Title => "Test Web Ada",
                             CSS_URI => "/css");
            end case;

            Page.Append_Code ("<div class='entete'>");
            Page.Append_Code ("<header><b>");
            Page.Append_Code ("<p class='title'>Welcome to DCOD-MONITOR</p>");
            Page.Append_Code ("<br/>");
            Page.Append_Code ("Listing of DCOD items");
            Page.Append_Code ("</b></header>");
            Page.Append_Code ("</div>");
            Page.Append_Code ("<br/>");
            Page.Append_Code ("<div class='corps'>");
            Page.Append_Code ("<table class='principale'>");
            Page.Append_Code ("<caption><h3>Process</h3></caption><br/>");
            Page.Append_Code ("<tr>");
            Page.Append_Code ("<th class='Colonnes'>Host|&|Types</th>");

            for Kind in Broadcast_Stuff.Process_Enum'Range loop
               case Kind is
                  when Broadcast_Stuff.Passive =>
                     null;
                  when others =>
                     Page.Append_Code ("<th class='Colonnes'>");
                     Page.Append_Code ("<p class='block'>");
                     case Kind is
                        when Broadcast_Stuff.Passive =>
                           null;
                        when Broadcast_Stuff.Posix_Memory_Handler =>
                           Page.Append_Code ("PMH");
                        when Broadcast_Stuff.Common_Transport_Layer =>
                           Page.Append_Code ("CTL");
                        when Broadcast_Stuff.ENX_Server =>
                           Page.Append_Code ("ENX SERV");
                        when Broadcast_Stuff.Launcher =>
                           Page.Append_Code ("LAUNCHER");
                        when Broadcast_Stuff.Sub_System_Coordinator =>
                           Page.Append_Code ("SUB SYSTEM COORD");
                        when Broadcast_Stuff.Aws_Shell =>
                           Page.Append_Code ("AWS SHELL");
                        when Broadcast_Stuff.Narval_Actor =>
                           Page.Append_Code ("NARVAL ACTOR");
                     end case;
                     Page.Append_Code ("</p></a></th>");
               end case;
            end loop;
            Page.Append_Code ("</tr>");
            for This_Entry of Entries loop
               case This_Entry.Kind is
                  when Broadcast_Stuff.Passive =>
                     null;
                  when others =>
                     if not Hosts.Contains (To_String (This_Entry.Host)) then

                        Hosts.Append (To_String (This_Entry.Host));

                     end if;
               end case;
            end loop;
            for Host of Hosts loop
               Page.Append_Code ("<tr>");
               Page.Append_Code ("<td>");
               Page.Append_Code (Host);
               Page.Append_Code ("</td>");
               for Kind in Broadcast_Stuff.Process_Enum'Range loop
                  case Kind is
                     when Broadcast_Stuff.Passive =>
                        null;
                     when others =>
                        Page.Append_Code ("<td>");
                        declare
                           Process_Present : Boolean := False;
                        begin
                           for This_Entry of Entries loop
                              if This_Entry.Kind = Kind and then This_Entry.Host = Host then
                                 Process_Present := True;
                              end if;
                           end loop;
                           if Process_Present then
                              Page.Append_Code ("<p><a href='/" & Kind'Img & "' target='_blank'><img src='" &
                                                  "https://cdn3.iconfinder.com/data/icons/free-3d-glossy-" &
                                                  "interface-icon-set/16/OK.png'/></a></p>");
                           else
                              Page.Append_Code ("<p><a href='' target='_blank'><img src='" &
                                                  "https://cdn1.iconfinder.com/data/icons/" &
                                                  "realistiK-new/16x16/actions/stop.png'/></a></p>");
                           end if;
                        end;
                        Page.Append_Code ("</td>");
                  end case;
               end loop;
            end loop;
            Page.Append_Code ("</tr>");
            Page.Append_Code ("</table>");
            Page.Append_Code ("<br/><br/><br/>");
            Page.Append_Code ("<form method='POST' action='/'>");
            Page.Append_Code ("<fieldset>");
            Page.Append_Code ("<legend>");
            Page.Append_Code ("Choix_Refresh :");
            Page.Append_Code ("</legend>");

            case Current_Mode is

               when Auto =>
                  Page.Append_Code ("<input type='radio' name='mode' value='auto'" &
                                      "checked='checked'>&nbsp;&nbsp;Automatically");
                  Page.Append_Code ("<input type='radio' name='mode' value='manual'>&nbsp;&nbsp;Manually");
               when Manual =>
                  Page.Append_Code ("<input type='radio' name='mode' value='auto'>&nbsp;&nbsp;Automatically");
                  Page.Append_Code ("<input type='radio' name='mode' value='manual'" &
                                      "checked='checked'>&nbsp;&nbsp;Manually");
            end case;

            Page.Append_Code ("<br/><br/><input type='Submit' value='Submit'>");
            Page.Append_Code ("<input type='Reset' value='Reset'>");
            Page.Append_Code ("</fieldset>");
            Page.Append_Code ("</form>");
            Page.Append_Code ("</div>");
            Page.Append_Code ("<div class='piedPage'>");
            Page.Append_Code ("<p class='TitlePied'>Support !</p></div>");
         end;
      elsif URI = "/css" then
         return AWS.Response.File (AWS.MIME.Text_CSS, DCOD.Environment.CSS_Directory.all & "/dcod_global.css");
      elsif URI = "/POSIX_MEMORY_HANDLER" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Posix_Memory_Handler);
      elsif URI = "/COMMON_TRANSPORT_LAYER" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Common_Transport_Layer);
      elsif URI = "/ENX_SERVER" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.ENX_Server);
      elsif URI = "/LAUNCHER" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Launcher);
      elsif URI = "/SUB_SYSTEM_COORDINATOR" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Sub_System_Coordinator);
      elsif URI = "/AWS_SHELL" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Aws_Shell);
      elsif URI = "/NARVAL_ACTOR" then
         Page := Format_Page.Redir_Page (Broadcast_Stuff.Narval_Actor);
      else
         Page.Malformed_URI (URI);
      end if;
      return AWS.Response.Build (AWS.MIME.Text_HTML, Page.Return_Page);
   end Dcod_Mr_CB;
end Dcod_Monitor_CB;
